<?php
namespace SIU\SQ\Lib\daos;

class dao_alarmas extends dao
{
	function obtener_alarmas( $filtro = array() )
	{
		$where = "";
		if(isset($filtro['id'])) {
			$where .= " AND fa.id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['codigo'])) {
			$where .= " AND fa.codigo = " . $this->quote($filtro['codigo']);
		}
		if(isset($filtro['descripcion'])) {
			$where .= " AND fa.descripcion = " . $this->quote($filtro['descripcion']);
		}
		if(isset($filtro['descripcion_comportamiento'])) {
			$where .= " AND fa.descripcion_comportamiento = " . $this->quote($filtro['descripcion_comportamiento']);
		}
		if(isset($filtro['estado'])) {
			$where .= " AND fa.estado = " . $this->quote($filtro['estado']);
		}
		
		$sql = "SELECT
					fa.id,
					fa.codigo,
					fa.descripcion,
					fa.id_tipo_facturador_alarma,
					fa.descripcion_comportamiento,
					fa.estado,
					fa.estado_actualizado_en,
					CASE tfa.parametros
						WHEN 'par_rango_inferior' THEN fap.par_rango_inferior
						WHEN 'par_rango_superior' THEN fap.par_rango_superior
						WHEN 'par_valor' THEN fap.par_valor
					END parametro_valor,
					row_to_json(tfa.*) tipos_facturador_alarmas,
					json_agg(fap.*) facturador_alarmas_parametros
				FROM
					facturador_alarmas fa
					LEFT JOIN tipos_facturador_alarmas tfa ON fa.id_tipo_facturador_alarma = tfa.id	
					LEFT JOIN facturador_alarmas_parametros fap	ON fa.id = fap.id_facturador_alarma
				WHERE 1=1
					$where 
					GROUP BY fa.id,fap.id,tfa.id
					ORDER BY fa.id;";
		$filas = $this->entorno()->db()->consultar($sql);
		//convierto los string json en arrays
		$resultado = [];
		foreach($filas as $f){
			$f['tipos_facturador_alarmas'] = $this->json_decode_caracteres_especiales($f['tipos_facturador_alarmas']);
			$f['facturador_alarmas_parametros'] = $this->json_decode_caracteres_especiales($f['facturador_alarmas_parametros']);
			$resultado[] = $f;
		}
		return $resultado;
	}
	
	function obtener_alarma($id)
	{
		$rs = $this->obtener_alarmas(['id' => $id]);
		if(count($rs) == 1){
			return $rs[0];
		}else{
			if(count($rs) > 1){
				throw new error_negocio("DAO_ALARMAS: "
						. "La consulta devuelve mas de un registro.");
			}else{
				return null;
			}
		}
	}
	
	function obtener_alarmas_configuradas($filtro = [])
	{
		$where = "";
		if(isset($filtro['id'])) {
			$where .= " AND id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['id_facturacion'])) {
			$where .= " AND id_facturacion = " . $this->quote($filtro['id_facturacion']);
		}
		$sql = "SELECT
					id,
					id_facturacion,
					id_facturador_alarma
				FROM
					facturaciones_alarmas_configuradas
				WHERE 1=1
					$where";
		return $this->entorno()->db()->consultar($sql);
	}
	
	function obtener_alarmas_generadas($filtro = [])
	{
		$where = "";
		if(isset($filtro['id'])) {
			$where .= " AND fa.id = " . $this->quote($filtro['id_facturador_alarma']);
		}
		if(isset($filtro['id_facturacion'])) {
			$where .= " AND sfag.id_facturacion = " . $this->quote($filtro['id_facturacion']);
		}
		//NOTA: la unica manera de anidar funciones de agregacion (json_agg()) es utilizando subqueries
		$sql = "SELECT
					fa.id,
					fa.codigo,
					fa.descripcion,
					fa.id_tipo_facturador_alarma,
					fa.descripcion_comportamiento,
					fa.estado,
					fa.estado_actualizado_en,
					CASE WHEN COUNT(sfag.*) > 0 THEN json_agg(sfag.*) ELSE '[]' END facturaciones_alarmas_generadas
				FROM
					facturador_alarmas fa
					LEFT JOIN (
						SELECT 
							fag.id,
							fag.id_facturacion,
							fag.id_facturador_alarma,
							json_agg(fagv.*) facturaciones_alarmas_generadas_ventas
						FROM
							facturaciones_alarmas_generadas fag
							JOIN facturaciones_alarmas_generadas_ventas fagv ON fag.id = fagv.id_facturacion_alarma_generada
						GROUP BY fag.id
					) sfag ON fa.id = sfag.id_facturador_alarma
					WHERE 1=1
						$where
					GROUP BY fa.id";
		$filas = $this->entorno()->db()->consultar($sql);
		//convierto los string json en arrays
		$resultado = [];
		foreach($filas as $f){
			$f['facturaciones_alarmas_generadas'] = $this->json_decode_caracteres_especiales($f['facturaciones_alarmas_generadas']);
			$resultado[] = $f;
		}
		return $resultado;
	}
	
	function obtener_alarmas_parametros($filtro = array())
	{
		$where = "";
		if(isset($filtro['id'])) {
			$where .= " AND id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['estado'])) {
			$where .= " AND estado = " . $this->quote($filtro['estado']);
		}
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					parametros,
					estado,
					estado_actualizado_en
				FROM
					tipos_facturador_alarmas
				WHERE 1=1
					$where";
		return $this->entorno()->db()->consultar($sql);
	}
}
