<?php
namespace SIU\SQ\Lib\daos;

use SIU\SQ\Lib\daos\dao;
use SIU\SQ\Lib\errores\error_negocio;

class dao_interoperabilidad extends dao
{
    public function get_sistema_externo($filtro)
    {
        $rs = $this->get_sistemas_externos($filtro);
        if (count($rs) == 1) {
            return $rs[0];
        } else {
            if (isset($filtro['token']) and count($rs) == 0) {
                throw new error_negocio("El sistema externo no se encuentra registrado");
            }
            if (count($rs) > 1) {
                throw new error_negocio("DAO_INTEROPERABILIDAD: get_sistema_externo: "
                    . "La consulta devolvio mas de un registro.");
            }
        }
    }

    public function esUuidValido($token)
    {
        $dato = !empty($token) && preg_match('/^\{?[A-Z0-9]{8}-[A-Z0-9]{4}-[A-Z0-9]{4}-[A-Z0-9]{4}-[A-Z0-9]{12}\}?$/i', $token);
        if (!$dato) {
            throw new error_negocio("El id ingresado no tiene formato UUID");
        }
    }

    public function get_sistemas_externos($filtro = array())
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND se.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['descripcion'])) {
            $where .= " AND se.descripcion = " . $this->quote($filtro['descripcion']);
        }
        if (isset($filtro['token'])) {
            $this->esUuidValido($filtro['token']);
            $where .= " AND token = " . $this->quote($filtro['token']);
        }
        
        $sql = "SELECT
					se.id,
					se.descripcion,
					se.token
				FROM
					sistemas_externos se
				WHERE 1=1 
					$where ORDER BY id;";
        
        return $this->entorno()->db()->consultar($sql);
    }

    public function get_sistemas_externos_recursos($filtro = array())
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND ser.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['sistema_externo'])) {
            $where .= " AND ser.sistema_externo = " . $this->quote($filtro['sistema_externo']);
        }
        if (isset($filtro['identificador_recurso'])) {
            $where .= " AND ser.identificador_recurso = " . $this->quote($filtro['identificador_recurso']);
        }

        $sql = "SELECT
					ser.id,
					ser.sistema_externo,
					ser.version_sis,
					ser.recurso,
					ser.identificador_recurso
				FROM
					sistemas_externos_recursos ser
				WHERE 1=1 
					$where ORDER BY id;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_sistema_externo_recurso($filtro)
    {
        $rs = $this->get_sistemas_externos_recursos($filtro);
        if (count($rs) == 1) {
            return $rs[0];
        } else {
            if (count($rs) == 0) {
                $msg = "El recurso asociado al sistema externo no se encuentra registrado";
                $this->entorno()->log()->error(__METHOD__ . ": " . $msg . ": Filtro: " . var_export($filtro, true));
                throw new error_negocio($msg);
            }
            if (count($rs) > 1) {
                $msg = "La consulta devolvio mas de un registro.";
                $this->entorno()->log()->error(__METHOD__ . ": " . $msg);
                throw new error_negocio($msg);
            }
        }
    }
}
