<?php

namespace SIU\SQ\Lib\daos;

use SIU\SQ\Lib\errores\error_negocio;

class dao_notificaciones extends dao
{
	function get($id)
	{
		$filtro['id'] = $id;		
		$rs = $this->get_lista($filtro);
		if(count($rs) > 1){
			throw new error_negocio(__METHOD__
					. "La consulta devolvio mas de un registro.");
		}else{
			return (isset($rs[0])) ? $rs[0] : NULL;
		}
	}

	function get_lista($filtro = [], $order_by = "", $limit = "", $additional_where = [])
	{
		$where = "";
		if(isset($filtro['id'])){
			$where .= " AND n.id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['id_tipo_notificacion'])){
			if (is_array($filtro['id_tipo_notificacion'])) {
				$where .= " AND n.id_tipo_notificacion in ( " . implode(',', $filtro['id_tipo_notificacion']) . ')';
			} else {
				$where .= " AND n.id_tipo_notificacion = " . $this->quote($filtro['id_tipo_notificacion']);
			}
		}
		if(isset($filtro['id_cliente'])){
			$where .= " AND n.par_id_cliente = " . $this->quote($filtro['id_cliente']);
		}
		if(isset($filtro['notificado_a__not_null'])){
			$where .= " AND n.notificado_a IS NOT NULL ";
		}
		if(isset($filtro['notificado_a__null'])){
			$where .= " AND n.notificado_a IS NULL ";
		}
		if(isset($filtro['enviada_por'])){
			$where .= " AND n.enviada_por ILIKE " . $this->quote("%" . $filtro['enviada_por'] . "%");
		}
		if(isset($filtro['fecha_desde'])){
			$where .= " AND date(n.enviada_en) >= " . $this->quote($filtro['fecha_desde']);
		}
		if(isset($filtro['fecha_hasta'])){
			$where .= " AND date(n.enviada_en) <= " . $this->quote($filtro['fecha_hasta']);
		}
		if(isset($filtro['enviada_en__null'])){
			$where .= " AND n.enviada_en IS NULL ";
		}
		if(isset($filtro['enviada_en__not_null'])){
			$where .= " AND n.enviada_en IS NOT NULL ";
		}
		if(isset($filtro['id_venta'])){
			$where .= " AND n.par_id_venta = " . $this->quote($filtro['id_venta']);
		}
		if(isset($filtro['id_servicio_suscripcion'])){
			$where .= " AND n.par_id_servicio_suscripcion = " . $this->quote($filtro['id_servicio_suscripcion']);
		}

		// Wheres que se agregan desde afuera. Para filtrar por perfil de datos desde afuera de lib.
		if (!empty($additional_where)) {
		    $where .= implode(' ', $additional_where);
        }

        // Se utiliza el * porque las columnas en nucleo y las unidades de venta difieren en los parametros...
		$sql = "SELECT
					n.*,
					p.nombres || ' ' || p.apellido cliente,
					row_to_json(tn.*) tipo_notificacion,
					v.leyenda venta,
					v.id_comprobante
				FROM
					notificaciones n
					JOIN tipos_notificaciones tn ON n.id_tipo_notificacion = tn.id
					LEFT JOIN ventas v ON n.par_id_venta = v.id
					LEFT JOIN (clientes c JOIN personas p ON c.id_persona = p.id)
					ON n.par_id_cliente = c.id
				WHERE
					1=1
					$where $order_by $limit;";
		
		$notificaciones = $this->entorno()->db()->consultar($sql);
		foreach(array_keys($notificaciones) as $k){
			$notificaciones[$k]['tipo_notificacion'] = $this->json_decode_caracteres_especiales($notificaciones[$k]['tipo_notificacion']);
		}
		return $notificaciones;
	}
	
	function get_tipo_notificacion($id)
	{
		$filtro['id'] = $id;		
		$rs = $this->get_lista_tipos_notificaciones($filtro);
		if(count($rs) > 1){
			throw new error_negocio(__METHOD__
					. "La consulta devolvio mas de un registro.");
		}else{
			return (isset($rs[0])) ? $rs[0] : NULL;
		}
	}

	function get_lista_tipos_notificaciones($filtro = array(), $order_by = "", $limit = "")
	{
		$where = "";
		if(isset($filtro['id'])){
			$where .= " AND id = " . $this->quote($filtro['id']);			
		}
		if(isset($filtro['codigo'])){
			$where .= " AND codigo = " . $this->quote($filtro['codigo']);
		}
		if(isset($filtro['descripcion'])){
			$where .= " AND descripcion = " . $this->quote($filtro['descripcion']);			
		}
		if(isset($filtro['tema'])){
			$where .= " AND tema = " . $this->quote($filtro['tema']);
		}
		if(isset($filtro['estado'])){
			$where .= " AND estado = " . $this->quote($filtro['estado']);
		}
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					momento_envio,
					rol,
					parametros,
					tema,
					mensaje,
					estado,
					estado_actualizado_en
				FROM
					tipos_notificaciones
				WHERE 1=1
					$where $order_by $limit;";
		return $this->entorno()->db()->consultar($sql);
	}
}
