<?php

namespace SIU\SQ\Lib\daos;

use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\constantes;

class dao_reloj_facturador extends dao
{
	function obtener_relojes($filtro = array())
	{
		$where = "";
		if(isset($filtro['id'])){
			$where .= " AND rf.id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['id_reloj_facturador_estado'])){
			if(is_array($filtro['id_reloj_facturador_estado'])){
				$where .= " AND rfe.id IN (" . implode(',', $this->quote($filtro['id_reloj_facturador_estado'])) . ")";
			}else {
				$where .= " AND rfe.id = " . $this->quote($filtro['id_reloj_facturador_estado']);
				}
		}
		if(isset($filtro['id_mes'])){
			$where .= " AND rf.id_mes = " . $filtro['id_mes'];
		}
		if(isset($filtro['id_anio'])){
			$where .= " AND rf.id_anio = " . $filtro['id_anio'];
		}
		if(isset($filtro['id_desde'])){
			$where .= " AND rf.id >= " . $this->quote($filtro['id_desde']);
		}
        if(isset($filtro['numero_mes'])){
            $where .= " AND m.numero_mes = " . $this->quote($filtro['numero_mes']);
        }
        if(isset($filtro['anio'])){
            $where .= " AND a.anio = " . $this->quote($filtro['anio']);
        }
		$sql = "SELECT
					rf.id,
					rf.codigo,
					rf.descripcion,
					rf.id_anio,
					rf.id_mes,
					rf.dia_inicio_facturacion,
					rf.dia_limite_facturacion,
					rf.id_reloj_facturador_estado,
					rf.facturador_ejecutado_en,
					rf.creado_por,
					rf.creado_en,
					rf.modificado_por,
					rf.modificado_en,
					rfe.descripcion estado,
					a.descripcion anio,
					m.descripcion mes
				FROM
					reloj_facturador rf,
					reloj_facturador_estados rfe,
					anios a,
					meses m
				WHERE
					rf.id_reloj_facturador_estado = rfe.id
					AND rf.id_anio = a.id
					AND rf.id_mes = m.id
					$where ORDER BY rf.id ASC;";
		return $this->entorno()->db()->consultar($sql);
	}
	
	function obtener_reloj($filtro)
	{
		$reloj = $this->obtener_relojes($filtro);
		if(!empty($reloj)){
			if(count($reloj) == 1){
				return $reloj[0];
			}else{
				throw new error_negocio("DAO_RELOJ_FACTURACION: Se ha encontrado mas de un reloj para la consulta.");
			}
		}
	}
	
	/**
	 * 
	 * @param integer $limit
	 * @return array periodos de facturacion futuros segun la fecha actual
	 * NOTA: si los periodos de facturacion no son mensuales esto deberia modificarse
	 */
	function obtener_siguientes_periodos_de_facturacion($limit)
	{
		$sql = "SELECT
					rf.id,
					rf.codigo,
					rf.descripcion,
					rf.id_anio,
					rf.id_mes,
					rf.dia_inicio_facturacion,
					rf.dia_limite_facturacion,
					rf.id_reloj_facturador_estado,
					rf.facturador_ejecutado_en,
					rf.creado_por,
					rf.creado_en,
					rf.modificado_por,
					rf.modificado_en
				FROM
					reloj_facturador rf
					JOIN anios a ON rf.id_anio = a.id
					JOIN meses m ON rf.id_mes = m.id
				WHERE
					a.anio > (SELECT EXTRACT(YEAR FROM CURRENT_DATE))
					OR (a.anio = (SELECT EXTRACT(YEAR FROM CURRENT_DATE))
					AND m.numero_mes > (SELECT EXTRACT(MONTH FROM CURRENT_DATE)))
					ORDER BY a.anio, m.numero_mes LIMIT $limit";
		
		return $this->entorno()->db()->consultar($sql);
	}
	
	function obtener_cantidad_de_relojes_posteriores_al_actual()
	{
		$filtro['id_reloj_facturador_estado'] = constantes::RELOJ_FACTURACION_ESTADO_ACTUAL;
		$reloj_actual = $this->obtener_reloj($filtro);
		
		$sql = "SELECT
					count(*) 
				FROM
					reloj_facturador rf 
				WHERE
					(id_anio = " . $reloj_actual['id_anio'] . "
					AND id_mes > " . $reloj_actual['id_mes'] . ")
					OR (id_anio > " . $reloj_actual['id_anio'] . ");";
		return $this->entorno()->db()->consultar($sql)[0]['count'];
	}
	
	function obtener_ultimo_reloj()
	{
		$sql = "SELECT
					rf.id,
					rf.codigo,
					rf.descripcion,
					rf.id_anio,
					rf.id_mes,
					rf.dia_inicio_facturacion,
					rf.dia_limite_facturacion,
					rf.id_reloj_facturador_estado,
					rf.facturador_ejecutado_en,
					rf.creado_por,
					rf.creado_en,
					rf.modificado_por,
					rf.modificado_en,
					a.anio numero_anio,
					m.numero_mes
				FROM
					reloj_facturador rf
					JOIN anios a ON rf.id_anio = a.id
					JOIN meses m ON rf.id_mes = m.id
				WHERE
					rf.id_anio = (SELECT max(id_anio)
									FROM reloj_facturador)
					AND rf.id_mes = (SELECT max(id_mes) 
										FROM reloj_facturador 
										WHERE id_anio = (SELECT max(id_anio) 
															FROM reloj_facturador));";
		return $this->entorno()->db()->consultar($sql)[0];
	}
	
	function obtener_anio($filtro)
	{
		$where = "";
		if(isset($filtro['numero_anio'])){
			$where .= " AND anio = " . $this->quote($filtro['numero_anio']);
		}
		if(isset($filtro['id'])){
			$where .= " AND id = " . $this->quote($filtro['id']);
		}
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					anio
				FROM
					anios
				WHERE 1=1
					$where";
		$r = $this->entorno()->db()->consultar($sql);
		return !empty($r) ? $r[0] : NULL;
	}
	
	function obtener_mes($filtro)
	{
		$where = "";
		if(isset($filtro['numero_mes'])){
			$where .= " AND numero_mes = " . $this->quote($filtro['numero_mes']);
		}
		if(isset($filtro['id'])){
			$where .= " AND id = " . $this->quote($filtro['id']);
		}
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					numero_mes
				FROM
					meses
				WHERE 1=1
					$where";
		$r = $this->entorno()->db()->consultar($sql);
		return !empty($r) ? $r[0] : NULL;
	}
	
	function get_reloj_facturador_estados($filtro = array(),$order_by = "ORDER BY codigo",$limit = "")
	{
		$where = "";
		if(isset($filtro['id'])){
			$where .= " AND id = " . $this->quote($filtro['id']);
		}
		if(isset($filtro['codigo'])){
			$where .= " AND codigo = " . $this->quote($filtro['codigo']);
		}
		if(isset($filtro['descripcion'])){
			$where .= " AND descripcion = " . $filtro['descripcion'];
		}
		if(isset($filtro['es_final'])){
			$where .= " AND es_final = " . $filtro['es_final'];
		}
		
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					es_final
				FROM
					reloj_facturador_estados
				WHERE 1=1
					$where $order_by $limit";
		return $this->entorno()->db()->consultar($sql);
	}
	
	function get_feriados($filtro = array())
	{
		$where = "";
		
		if(isset($filtro['1er_vencimiento'])){
			if(is_array($filtro['1er_vencimiento'])){
				$where .= " OR mes = " . $this->quote($filtro['1er_vencimiento']['mes']);
				$where .= " AND anio = " . $this->quote($filtro['1er_vencimiento']['anio']);
				
			}
		}
		
		if(isset($filtro['2do_vencimiento'])){
			if(is_array($filtro['2do_vencimiento'])){
				$where .= " OR mes = " . $this->quote($filtro['2do_vencimiento']['mes']);
				$where .= " AND anio = " . $this->quote($filtro['2do_vencimiento']['anio']);
				
			}
		}
		
		$sql = "SELECT dia 
				FROM feriados 
				WHERE 1=1
					$where";
						
		return $this->entorno()->db()->consultar($sql);
	}
}
