<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace SIU\SQ\Lib\entorno;

/**
 * UniversalClassLoader implements a "universal" autoloader for PHP 5.3.
 *
 * It is able to load classes that use either:
 *
 **********************************************************************************
 **********************************************************************************
 *The technical interoperability standards for PHP 5.3 namespaces and
 *    class names (http://groups.google.com/group/php-standards/web/psr-0-final-proposal);
 * SE USA ESTE ESQUEMA SALVO LO DE LOS _ EN LOS NOMBRES DE LAS CLASES
 **********************************************************************************
 **********************************************************************************
 *  * The PEAR naming convention for classes (http://pear.php.net/).
 *
 * Classes from a sub-namespace or a sub-hierarchy of PEAR classes can be
 * looked for in a list of locations to ease the vendoring of a sub-set of
 * classes for large projects.
 *
 * Example usage:
 *
 *     $loader = new UniversalClassLoader();
 *
 *     // register classes with namespaces
 *     $loader->registerNamespaces(array(
 *         'Symfony\Component' => __DIR__.'/component',
 *         'Symfony'           => __DIR__.'/framework',
 *         'Sensio'            => array(__DIR__.'/src', __DIR__.'/vendor'),
 *     ));
 *
 *     // register a library using the PEAR naming convention
 *     $loader->registerPrefixes(array(
 *         'Swift_' => __DIR__.'/Swift',
 *     ));
 *
 *     // activate the autoloader
 *     $loader->register();
 *
 * In this example, if you try to use a class in the Symfony\Component
 * namespace or one of its children (Symfony\Component\Console for instance),
 * the autoloader will first look for the class under the component/
 * directory, and it will then fallback to the framework/ directory if not
 * found before giving up.
 *
 * @author Fabien Potencier <fabien@symfony.com>
 *
 * @api
 */
class UniversalClassLoader
{
    private $namespaces = array();

    public function getNamespaces()
    {
        return $this->namespaces;
    }

    public function registerNamespace($namespace, $paths, $alias=array())
    {
        $this->namespaces[$namespace] = array('path'=>$paths,'alias'=>$alias);
    }
	
    public function register($prepend = false)
    {
        spl_autoload_register(array($this, 'loadClass'), true, $prepend);
    }


    public function loadClass($class)
    {
        if ($file = $this->findFile($class)) {
            require $file;
		}
    }

    public function findFile($class)
    {
        if ('\\' == $class[0]) {
            $class = substr($class, 1);
        }
        if (false !== $pos = strrpos($class, '\\')) {
            // namespaced class name
            $namespace = substr($class, 0, $pos);
            foreach ($this->namespaces as $ns => $dirs) {
				$dir = $dirs['path'];
				if (0 === strpos($namespace, $ns)) {
					$className = substr($class, $pos + 1);
					if(!empty($dirs['alias'])){
						$namespace = str_replace($dirs['alias'][0],
													$dirs['alias'][1],
												$namespace);
					}
					$file = $dir. DIRECTORY_SEPARATOR .
								str_replace('\\', DIRECTORY_SEPARATOR, $namespace).
								DIRECTORY_SEPARATOR.$className.'.php';
					echo "FILE: $file\n";
					if (file_exists($file)) {
						return $file;
					}
				}
            }
        }
    }
}

