<?php
namespace SIU\SQ\Lib;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\entorno\configurador;

class lib
{
    static protected $entorno;

    static function set_entorno($entorno)
    {
        self::$entorno = $entorno;
    }

    static function entorno()
    {
        if (!isset(self::$entorno)){
            throw new error("Entorno no configurado");
        }
        return self::$entorno;
    }

    /**
     *
     * @return procesos\catalogo_procesos
     */
    static function catalogo_procesos($entorno=null)
    {
        if(!isset($entorno)) {
            $entorno = self::entorno();
        }
        return new procesos\catalogo($entorno);
    }

    /**
     *
     * @return modelo\catalogo
     */
    static function catalogo_modelo($entorno=null)
    {
        if(!isset($entorno)) {
            $entorno = self::entorno();
        }
        return new modelo\catalogo($entorno);
    }

    /**
     *
     * @return daos\catalogo
     */
    static function catalogo_daos($entorno=null)
    {
        if(!isset($entorno)) {
            $entorno = self::entorno();
        }
        return new daos\catalogo($entorno);
    }

    /**
     *
     * @return sistemas_externos\catalogo
     */
    static function catalogo_sistemas_externos($entorno=null)
    {
        if(!isset($entorno)) {
            $entorno = self::entorno();
        }
        return new sistemas_externos\catalogo($entorno);
    }

    /**
     *
     * @return util\parametros_sistema
     */
    static function parametros_sistema($entorno=null)
    {
        if(!isset($entorno)) {
            $entorno = self::entorno();
        }
        return new util\parametros_sistema($entorno);
    }
	
	static function mailer()
	{
		$config = new configurador();
		$host = $config->sq_ini['mailer']['SMTP_HOST'];
		$smtp_auth = $config->sq_ini['mailer']['SMTP_AUTENTICAR'];
		$username = $config->sq_ini['mailer']['SMTP_USUARIO'];
		$password = $config->sq_ini['mailer']['SMTP_CLAVE'];
		$SMTPSecure = $config->sq_ini['mailer']['SMTP_PROTOCOLO_SEGURIDAD'];
		$port = $config->sq_ini['mailer']['SMTP_PUERTO'];
		$es_desarrollo = $config->sq_ini['mailer']['ES_DESARROLLO'];
		$envia_emails = $config->sq_ini['mailer']['ENVIA_EMAILS'];
		$email_envios_desarrollo = $config->sq_ini['mailer']['EMAIL_ENVIOS_DESARROLLO'];
		$mailer = new util\mailer($host,$smtp_auth,$username,$password,$SMTPSecure,$port,$es_desarrollo,$envia_emails,$email_envios_desarrollo);
		if(isset($config->sq_ini['mailer']['SMTP_AUTO_TLS'])){
			switch ($config->sq_ini['mailer']['SMTP_AUTO_TLS']){
				case '0':
					$mailer->set_smtp_auto_tls(false);
					break;
				case '1':
					$mailer->set_smtp_auto_tls(true);
					break;
			}
		}
		return $mailer;
	}
    
    static function rest_hooks()
    {
        return \SIU\SQ\RestHooks\rest_hooks::getRestMessenger();
    }
    
    static function es_modo_mantenimiento()
    {
        $config = new configurador();
        $dir = $config->get_config_dir();
        $proyecto = parse_ini_file($dir . "/../toba/proyecto.ini", TRUE);
        return $proyecto['proyecto']['modo_mantenimiento'];
    }
	
	/**
	 * Retorna una instancia de un generador de pdf que toma la data de un json
	 * @return \SIU\SQ\Lib\util\toba_vista_jasperreports_json
	 */
	static function toba_vista_jasperreports_json($nombre = "comprobante_")
	{
		return new util\toba_vista_jasperreports_json($nombre);
	}

    /**
     * Retorna el nombre del esquema si existe y false si no existe
     * @return mixed
     */
    public static function existe_esquema_auditoria($modulo)
    {
        $esquema = self::get_esquema_auditoria($modulo);
        $sql = "SELECT schema_name FROM information_schema.schemata WHERE schema_name = '" . $esquema . "'";

        return (!empty(self::entorno()->db()->consultar($sql))) ? $esquema : false;
    }

    /**
     * Retorna el nombre del esquema si existe y die si no existe
     * @return string
     */
    public static function get_esquema_auditoria($modulo)
    {
        $config = new configurador();
        if (isset($config->sq_ini[$modulo])) {
            $parametros_db = $config->sq_ini[$modulo]['db'];
            $def_base = $config->bases_ini[$parametros_db];
            return $def_base['schema'] . "_auditoria";
        } else {
            die("El módulo '" . $modulo . "' no existe en la definición de sq.ini.");
        }
    }

}
