<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\errores\error_not_found;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\lib;

class 	comprobante extends modelo
{
	protected function inicializar()
	{
		$this->datos['id']							= null;
		$this->datos['id_comprobante_template']		= null;
        $this->datos['numero_comprobante']			= null;
        $this->datos['presentacion']				= null;
        $this->datos['importe_control']				= null;
        $this->datos['id_comprobante_relacionado']	= null;
        $this->datos['id_rdi']						= null;
        $this->datos['uuid_arai_documentos']	    = null;
        $this->datos['id_punto_cobranza']			= null;
		$this->datos['estado']						= constantes::COMPROBANTE_ESTADO_PENDIENTE;
		$this->datos['estado_actualizado_en']		= "now()";
	}

	protected function cargar($id_comprobante)
	{
		$comprobante = lib::catalogo_daos()->comprobantes()->get( $id_comprobante );
		if(!empty($comprobante)) {
			$this->datos = $comprobante;
		} else {
			throw new error_not_found("Comprobante: El comprobante que intenta cargar no existe.");
		}
	}

	protected function validar()
	{
		$campos_obligatorios = array(
			'id_comprobante_template',
			'estado'
		);

		$this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __CLASS__);

		// validar número de comprobante único
		if(!is_null($this->datos['numero_comprobante']) && $this->datos['numero_comprobante'] != '')
		{
			$filtro = [
				'numero_comprobante' => $this->get_numero_comprobante()
			];
			// Si existe UV, lo agrego como condicion para aceptar un mismo número de comprobante para distintas UVs
            if ($this->datos['id_punto_cobranza']) {
                $filtro['id_punto_cobranza'] = $this->datos['id_punto_cobranza'];
			}
            if ($this->datos['id_comprobante_template']) {
                $filtro['id_comprobante_template'] = $this->datos['id_comprobante_template'];
            }
			$comprobantes = lib::catalogo_daos()->comprobantes()->get_lista($filtro);
			foreach ($comprobantes as $comprobante) {
				if ($comprobante['id'] != $this->datos['id']) {
					throw new error_negocio("Comprobante: El número de comprobante " . $this->get_numero_comprobante() . " ya existe.");
				}
			}
		}
	}

	public function guardar()
    {
        $this->validar();

        if($this->datos['id'] !== null) {
            $this->actualizar_comprobante();
        } else {
            $this->grabar_comprobante();
        }
		return $this->datos['id'];
    }

	public function cambiar_estado($estado)
	{
		$this->validar_cambio_de_estado($estado);
		$this->set_estado($estado);
	}

	protected function validar_cambio_de_estado($estado)
	{
		switch ($estado) {
			case constantes::COMPROBANTE_ESTADO_PENDIENTE:
			case constantes::COMPROBANTE_ESTADO_FINALIZADO:
				if($this->get_estado() == constantes::COMPROBANTE_ESTADO_ANULADO) {
					throw new error_negocio("Comprobante: No puede cambiar el estado del comprobanda a Generado ya que este se encuentra anulado.");
				}
				break;
			case constantes::COMPROBANTE_ESTADO_ANULADO:
				break;
			default:
				throw new error_negocio("Comprobante: El estado que intenta guardar no es valido.");
				break;
		}
	}

	//---- MÉTODOS DE GRABACIÓN ----//

	protected function grabar_comprobante()
	{
		$datos = array(
			'id_comprobante_template' 		=> $this->datos['id_comprobante_template'],
			'numero_comprobante'			=> $this->datos['numero_comprobante'],
			'presentacion'					=> json_encode($this->datos['presentacion'],JSON_UNESCAPED_UNICODE),
			'importe_control'				=> $this->datos['importe_control'],
			'id_comprobante_relacionado'	=> $this->datos['id_comprobante_relacionado'],
			'id_rdi'						=> $this->datos['id_rdi'],
			'uuid_arai_documentos'	    	=> $this->datos['uuid_arai_documentos'],
            'id_punto_cobranza'				=> $this->datos['id_punto_cobranza'],
			'estado'						=> $this->datos['estado'],
			'estado_actualizado_en'			=> $this->datos['estado_actualizado_en']
		);
		if(json_last_error() != JSON_ERROR_NONE){
			$msg = "Comprobante: Error la consumir el JSON del campo 'presentacion'";
			lib::entorno()->log()->error(__METHOD__ . ": Error json (" . json_last_error() . ")" . $msg);
			throw new error_negocio($msg);
		}

		$sql = sql::sql_array_a_insert('comprobantes', $datos, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
		$seq = $this->entorno()->db()->consultar_fila("SELECT currval('comprobantes_id_seq') as seq;");
		$this->datos['id'] = $seq['seq'];
	}

	protected function actualizar_comprobante()
	{
		$datos = array(
			'numero_comprobante'		=> $this->datos['numero_comprobante'],
			'presentacion'				=> json_encode($this->datos['presentacion'],JSON_UNESCAPED_UNICODE),
            'uuid_arai_documentos'	    => $this->datos['uuid_arai_documentos'],
			'estado'					=> $this->datos['estado'],
			'estado_actualizado_en'		=> $this->datos['estado_actualizado_en']
		);
		$where = ['id' => $this->datos['id']];
		$sql_update = sql::sql_array_a_update('comprobantes', $datos, $where, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql_update);
	}

	//---- GETTERS Y SETTERS ----//

    public function get_id()
    {
        return $this->datos['id'];
    }

    public function get_id_comprobante_template()
    {
        return $this->datos['id_comprobante_template'];
    }

    public function get_numero_comprobante()
    {
        return $this->datos['numero_comprobante'];
    }

    public function get_presentacion()
    {
        return $this->datos['presentacion'];
    }

    public function get_importe_control()
    {
        return $this->datos['importe_control'];
    }

    public function get_id_comprobante_relacionado()
    {
        return $this->datos['id_comprobante_relacionado'];
    }

    public function get_id_rdi()
    {
        return $this->datos['id_rdi'];
    }

    public function get_uuid_arai_documentos()
    {
        return $this->datos['uuid_arai_documentos'];
    }

    public function get_id_punto_cobranza()
    {
        return $this->datos['id_punto_cobranza'];
    }

	public function get_estado()
    {
        return $this->datos['estado'];
    }

    public function get_estado_actualizado_en()
    {
        return $this->datos['estado_actualizado_en'];
    }

	public function set_id_comprobante_template($id_comprobante_template)
    {
        $this->datos['id_comprobante_template'] = $id_comprobante_template;
    }

	public function set_numero_comprobante($numero_comprobante)
    {
        $this->datos['numero_comprobante'] = $numero_comprobante;
    }

	public function set_presentacion($presentacion)
    {
        $this->datos['presentacion'] = $presentacion;
    }

	public function set_importe_control($importe_control)
    {
        $this->datos['importe_control'] = $importe_control;
    }

	public function set_id_comprobante_relacionado($id_comprobante_relacionado)
    {
        $this->datos['id_comprobante_relacionado'] = $id_comprobante_relacionado;
    }

	public function set_id_rdi($id_rdi)
    {
        $this->datos['id_rdi'] = $id_rdi;
    }

	public function set_uuid_arai_documentos($uuid_arai_documentos)
    {
        $this->datos['uuid_arai_documentos'] = $uuid_arai_documentos;
    }

    public function set_id_punto_cobranza($id_punto_cobranza)
    {
        $this->datos['id_punto_cobranza'] = $id_punto_cobranza;
    }

	public function set_estado($estado)
    {
        $this->datos['estado'] = $estado;
    }

	public function set_estado_actualizado_en($estado_actualizado_en)
    {
        $this->datos['estado_actualizado_en'] = $estado_actualizado_en;
    }


}
