<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\lib\lib;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\errores\error_not_found;

class comprobante_template extends modelo
{
	protected function inicializar()
	{
		$this->datos['id'] = null;
		$this->datos['codigo'] = null;
		$this->datos['descripcion'] = null;
		$this->datos['tipo_comprobante'] = null;
		$this->datos['modelo'] = null;
		$this->datos['numeracion'] = null;
		$this->datos['numeracion_externa'] = false;
		$this->datos['estado'] = null;
		$this->datos['estado_actualizado_en'] = null;
	}
	
	protected function cargar($id)
	{		
		$this->datos = lib::catalogo_daos()->comprobantes()->get_template($id);
		if(empty($this->datos)){
			throw new error_not_found(__METHOD__ . " No se pudo recuperar el comprobante_template ID $id");
		}
	}	

	protected function validar()
	{
		$campos_obligatorios = ['codigo','descripcion','tipo_comprobante','modelo'];
		$this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __METHOD__);
	}

	public function guardar()
	{
		$this->validar();		
		if(empty(lib::catalogo_daos()->comprobantes()->get_template($this->get_id()))) {
			$this->grabar();
		} else {
			$this->actualizar();
		}
		return $this->get_id();
	}
	
	private function grabar()
	{
		$datos = [
			'id'					=> $this->get_id(),
			'codigo'				=> $this->datos['codigo'],
			'descripcion'			=> $this->datos['descripcion'],
			'tipo_comprobante'		=> $this->datos['tipo_comprobante'],
			'modelo'				=> $this->datos['modelo'],
			'numeracion'			=> $this->datos['numeracion'],
			'numeracion_externa'	=> $this->datos['numeracion_externa'],
			'estado'				=> $this->datos['estado'],
			'estado_actualizado_en'	=> $this->datos['estado_actualizado_en']
		];
		$sql = sql::sql_array_a_insert('comprobantes_templates', $datos, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
	}
	
	private function actualizar()
	{
		$datos = [
			'codigo'				=> $this->datos['codigo'],
			'descripcion'			=> $this->datos['descripcion'],
			'tipo_comprobante'		=> $this->datos['tipo_comprobante'],
			'modelo'				=> $this->datos['modelo'],
			'numeracion'			=> $this->datos['numeracion'],
            'numeracion_externa'	=> $this->datos['numeracion_externa'],
            'estado'				=> $this->datos['estado'],
			'estado_actualizado_en'	=> $this->datos['estado_actualizado_en']
		];
		$where = ['id' => $this->get_id()];
		$sql = sql::sql_array_a_update('comprobantes_templates', $datos, $where, $this->entorno()->db());		
		$this->entorno()->db()->ejecutar($sql);
	}
	
	function avanzar_numeracion()
	{
		if($this->datos['numeracion_externa'] == false) {
			if (empty($this->datos['numeracion'])) {
				$this->datos['numeracion'] = 1;
			} else {
				$this->datos['numeracion'] = $this->datos['numeracion'] + 1;
			}
			return $this->datos['numeracion'];
		} else {
			throw new error_negocio(__METHOD__ . " No se pudo generar el proximo número de comprobante ya que este posee numeración externa.");
		}
	}
	
	//--SETTERS
	
	function set_id($id)
	{
		$this->datos['id'] = $id;
	}
	
	function set_codigo($codigo)
	{
		$this->datos['codigo'] = $codigo;
	}
	
	function set_descripcion($descripcion)
	{
		$this->datos['descripcion'] = $descripcion;
	}
	function set_tipo_comprobante($tipo_comprobante)
	{
		$this->datos['tipo_comprobante'] = $tipo_comprobante;
	}
	
	function set_modelo($modelo)
	{
		$this->datos['modelo'] = $modelo;
	}
	
	function set_numeracion($numeracion)
	{
		if($this->datos['numeracion_externa'] == true) {
			$this->datos['numeracion'] = $numeracion;
		} else {
			throw new error_negocio(__METHOD__ . " No se puede setear el número de comprobante ya que el template posee numeración externa.");
		}
	}
	
	function set_estado($estado)
	{
		if(is_null($this->datos['estado']) || $this->datos['estado'] != $estado){
			$this->datos['estado_actualizado_en'] = 'now()';
		}
		$this->datos['estado'] = $estado;
	}

    function set_numeracion_externa($numeracion_externa)
    {
        $this->datos['numeracion_externa'] = $numeracion_externa;
    }

    //--GETTERS
	
	function get_numeracion()
	{
		return $this->datos['numeracion'];
	}
	
	function tiene_numeracion_externa()
	{
		return $this->datos['numeracion_externa'];
	}
}
