<?php

namespace  SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Uv\sq_uv;
use SIU\SQ\Uv\modelo\venta;
use SIU\SQ\Uv\constantes;

class facturador extends modelo
{
    /*
     *  TEMAS a PENSAR
     *
     * Sistema de avisos y alertas?
     *
     * Puntos de extension
     *  - informar al sistema de alertas
     *  - Establecer alguna regla de negocio especial
     *  - calculador de moras?
     *
     */

    protected $calculadora_de_mora;
    protected $reloj_facturador;
    protected $id_reloj;
    protected $dias_vencimiento;
    protected $ids_servicios_instancias = array();
	
    protected $cantidad_ventas_pre_generadas;
    protected $importe_ventas_pre_generadas;
    protected $ventas_generadas = [];
    protected $cantidad_ventas_generadas;
    protected $importe_ventas_generadas;
    protected $cantidad_ventas_cobradas;
    protected $importe_ventas_cobradas;
	
    protected $id_facturacion;
	protected $ids_facturador_alarmas;

	protected $proceso;

	protected function inicializar()
	{
		// Obtener el reloj activo
        $this->reloj_facturador = lib::catalogo_modelo()->reloj();
        $this->id_reloj = $this->reloj_facturador->get_actual();
		$this->cantidad_ventas_pre_generadas = 0;
		$this->importe_ventas_pre_generadas = 0;
		$this->cantidad_ventas_generadas = 0;
		$this->importe_ventas_generadas = 0;
		$this->cantidad_ventas_cobradas = 0;
		$this->importe_ventas_cobradas = 0;
		$this->id_facturacion = null;
		$this->ids_facturador_alarmas = [];
		//Nota: cuando se crea una venta, el campo ventas.cobranza_vence_en se setea con la fecha
		//      del proceso + el valor del parametros_sistema("DIAS_1er_V"). VER SI ES FERIADO????
		//      
        // SI el parametro '"APLIC_MORA"' esta en 'S' 
		//		$this->calculadora_de_mora = new calculadora_de_mora();
	}

	public function set_proceso($proceso)
	{
		$this->proceso = $proceso;
	}
	
	protected function validar()
	{
		if(empty($this->ids_servicios_instancias)){
			throw new error_negocio("No se especificaron servicios para facturar.");
		}
	}
	
	public function guardar()
	{
		throw new error_negocio("FACTURADOR: Utilizar el metodo 'Generar' o 'Confirmar' para completar una acción de facturación.");
	}
	

    public function generar()
    {
		$this->validar();
		$this->generar_cabecera_facturacion();
		$this->guardar_instancias_a_facturar();
		if(!empty($this->ids_facturador_alarmas)){
			$this->guardar_facturaciones_alarmas_configuradas();
		}
		try {
			$this->programar_cuotas_continuas();
			$this->procesar_ventas_vencidas();
			$this->generar_ventas_por_suscripciones();
	        $this->procesar_ventas_por_suscripcion_y_cliente();
			$this->contar_ventas_pregeneradas();
		} catch (error_negocio $e) {
			lib::entorno()->log()->error("Facturador: Error al generar la facturacion: " . $e->getMessage());
			$this->revertir();
			throw $e;
		}
    }
	
	function contar_ventas_pregeneradas()
	{
		// Obtener las ventas en estado VENTA_ESTADO_PRE_FACT
		$filtro['id_venta_estado'] = constantes::VENTA_ESTADO_PRE_FACT;
		$filtro['id_periodo_facturacion'] = $this->id_reloj;
		$ventas_prefacturadas = sq_uv::catalogo_daos()->ventas()->get_ventas($filtro);
		$ids_ventas_agrupadas_en_sumarizada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_sumarizadas(),'id_venta_sumarizada');
        $ids_ventas_tipo_confinanciada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_cofinanciadas(),'id');
		foreach($ventas_prefacturadas as $venta_prefact)
		{
			// Verificar si la venta no es parte de una sumarizada
			if(!in_array($venta_prefact['id'], array_merge($ids_ventas_agrupadas_en_sumarizada, $ids_ventas_tipo_confinanciada))){
				$this->cantidad_ventas_pre_generadas++;
				$this->importe_ventas_pre_generadas += $venta_prefact['importe_total'];
			}
		}
	}

    public function confirmar()
    {
		try {
            // Confirmar el procesamiento de los servicios instancias
            $this->cambiar_estado_facturacion_servicios_instancias(constantes::FACTURADOR_SERVICIO_INSTANCIA_ESTADO_CONFIRMADO);
            
            // Obtener las ventas en estado VENTA_ESTADO_PRE_FACT
            $filtro['id_venta_estado'] = constantes::VENTA_ESTADO_PRE_FACT;
            $filtro['id_periodo_facturacion'] = $this->id_reloj;
            $ventas_prefacturadas = sq_uv::catalogo_daos()->ventas()->get_ventas($filtro, null, null, "cobranza_vence_en", null, null, true);

			$this->cambiar_estado_ventas($ventas_prefacturadas, constantes::VENTA_PENDIENTE);

            lib::entorno()->log()->debug(__METHOD__, ["Cantidad de ventas en estado de prefacturación cambiadas de estado" => count($ventas_prefacturadas)]);

			$this->notificar_ventas_al_nucleo($ventas_prefacturadas);

            lib::entorno()->log()->debug(__METHOD__, ["Cantidad de ventas en estado de prefacturación notificadas al nucleo" => count($ventas_prefacturadas)]);

			// Si no queda ninguna instancia para facturar en el id de reloj actual, avanzo el facturador
			$datos = sq_uv::catalogo_daos()->servicios()->get_servicios_instancias_no_facturados($this->id_reloj);
			if(empty($datos)) {
				$this->reloj_facturador->avanzar();
			}
			$this->eliminar_log_reversion();
		} catch (error_negocio $e) {
			$this->revertir_confirmacion($ventas_prefacturadas);
			throw $e;
		}
    }
	
	protected function revertir_confirmacion($ventas_prefacturadas)
	{
        $this->cambiar_estado_ventas($ventas_prefacturadas, constantes::VENTA_ESTADO_PRE_FACT);


        $msg = 'FACTURADOR: REVERTIR CONFIRMACION: NO IMPLEMENTADO!';

	    lib::entorno()->log()->critical(__METHOD__, [$msg]);

		ei_arbol(array(), $msg);
	}
	
	function get_id_reloj()
	{
		return $this->id_reloj;
	}
	
	public function get_cantidad_ventas_pre_generadas()
	{
		return $this->cantidad_ventas_pre_generadas;
	}
	
	public function get_importe_ventas_pre_generadas()
	{
		return $this->importe_ventas_pre_generadas;
	}
	
	public function get_cantidad_ventas_generadas()
	{
		return $this->cantidad_ventas_generadas;
	}
	
	public function get_ventas_generadas()
	{
		return $this->ventas_generadas;
	}
	
	public function get_importe_ventas_generadas()
	{
		return $this->importe_ventas_generadas;
	}
	
	public function get_cantidad_ventas_cobradas()
	{
		return $this->cantidad_ventas_cobradas;
	}
	
	public function get_importe_ventas_cobradas()
	{
		return $this->importe_ventas_cobradas;
	}
	
	public function set_dia_primer_vencimiento($id_reloj_1er_vencimiento, $dia_1er_vencimiento)
	{
		$this->dias_vencimiento['dia_1er_vencimiento'] = $dia_1er_vencimiento;
		$this->dias_vencimiento['id_reloj_1er_vencimiento'] = $id_reloj_1er_vencimiento;
	}
	
	public function set_dia_segundo_vencimiento($id_reloj_2do_vencimiento, $dia_2do_vencimiento)
	{
		$this->dias_vencimiento['dia_2do_vencimiento'] = $dia_2do_vencimiento;
		$this->dias_vencimiento['id_reloj_2do_vencimiento'] = $id_reloj_2do_vencimiento;
	}
	
	public function set_instancias_a_facturar($ids_servicios_instancias)
	{
		$this->ids_servicios_instancias = $ids_servicios_instancias;
	}
	
// Obsoleto, Borrar??????
	public function set_id_facturacion($id_facturacion)
	{
		$this->id_facturacion = $id_facturacion;
	}
	
	public function get_id_facturacion()
	{
		return $this->id_facturacion;
	}
	
	public function set_ids_facturador_alarmas(array $ids_facturador_alarmas)
	{
		$this->ids_facturador_alarmas = $ids_facturador_alarmas;
	}

    public function revertir()
    {
		$this->revertir_facturacion();
		
		$this->eliminar_facturaciones_alarmas_configuradas();
		$this->eliminar_facturaciones_alarmas_generadas();
		$this->eliminar_instancias_a_facturar();
		$this->eliminar_cabecera_facturacion();
		
    }

    //-----------------------------------------------------------
    // GENERACION
    //-----------------------------------------------------------
	
	protected function generar_cabecera_facturacion()
	{
		if(!isset($this->dias_vencimiento['dia_1er_vencimiento']) AND !isset($this->dias_vencimiento['id_reloj_1er_vencimiento'])){
			throw new error_negocio("No se definieron las fechas de vencimiento para el facturador.");
		}
		if(lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_VENCIMIENTOS) == constantes::SEGUNDO_VENCIMIENTO) {
			if(!isset($this->dias_vencimiento['dia_2do_vencimiento']) AND !isset($this->dias_vencimiento['id_reloj_2do_vencimiento'])){
				throw new error_negocio("No se definieron las fechas de vencimiento para el facturador.");
			}
			$datos['id_reloj_facturador_2do_venc']	= $this->dias_vencimiento['id_reloj_2do_vencimiento'];
			$datos['dia_2do_vencimiento']			= $this->dias_vencimiento['dia_2do_vencimiento'];
		}
		$datos['id_ejecucion_proceso']			= $this->proceso->get_id_ejecucion();
		$datos['id_reloj_facturador_1er_venc'] 	= $this->dias_vencimiento['id_reloj_1er_vencimiento'];
		$datos['dia_1er_vencimiento']			= $this->dias_vencimiento['dia_1er_vencimiento'];
		$datos['id_reloj_facturador']			= lib::catalogo_modelo()->reloj()->get_actual();
		$sql = sql::sql_array_a_insert("facturaciones", $datos, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
		// Obtener el id de facturación
		$this->id_facturacion = $this->entorno()->db()->ultimo_insert_id("facturaciones_id_seq");
	}
	
	protected function guardar_instancias_a_facturar()
	{
		$sql = null;
		foreach($this->ids_servicios_instancias as $instancia){
			$datos['id_servicio_instancia']	= $instancia;
			$datos['id_facturacion']		= $this->id_facturacion;
			$datos['estado']		        = constantes::FACTURADOR_SERVICIO_INSTANCIA_ESTADO_INICIADO;
			$sql .= sql::sql_array_a_insert("facturaciones_servicios_instancias", $datos, $this->entorno()->db());
		}
		if($sql) {
			$this->entorno()->db()->ejecutar($sql);
		}else{
			throw new error_negocio("FACTURADOR: No se encontraron servicios instancias para procesar.");
		}
	}
	
	protected function guardar_facturaciones_alarmas_configuradas()
	{
		$sql = null;
		foreach($this->ids_facturador_alarmas as $id){
			$datos['id_facturador_alarma'] = $id;
			$datos['id_facturacion'] = $this->id_facturacion;
			$sql .= sql::sql_array_a_insert("facturaciones_alarmas_configuradas", $datos, $this->entorno()->db());
		}
		if($sql){
			$this->entorno()->db()->ejecutar($sql);
		}
	}

    protected function programar_cuotas_continuas()
    {
        /*
         * Iterar suscripciones ACTIVAS
         *      SI $this->id_periodo_facturacion actual no foma parte de una excepcion de la instancia
         *           SI el programa de venta seleccionado por el cliente, tiene un articulo con frecuencia SIEMPRE,
         *              crear registo en "Servicios prestaciones pre programadas"
         *                  - con id_periodo_factuacion = $this->id_periodo_facturacion
         *                  - marcar como creado_por_facturador en TRUE
         */
		//-- Obtener las suscripciones ACTIVAS, FUERA PERIODO EXCEPCION, FRECUENCIA SIEMPRE,
		$filtro['ids_servicios_instancias'] = $this->ids_servicios_instancias;
		$suscripciones = sq_uv::catalogo_daos()->servicios()->get_suscripciones_cuotas_continuas($filtro);
		foreach($suscripciones as $suscripcion){
			$suscripcion_obj = sq_uv::catalogo_modelo()->suscripcion($suscripcion['id']);
			$suscripcion_obj->set_creado_por_facturador(TRUE);
			$prog_venta = sq_uv::catalogo_modelo()->programacion_venta()->get_detalle( $suscripcion['id_servicio_instancia_prog_venta'] );
			if(isset($prog_venta['detalle']) AND count($prog_venta['detalle']) == 1){
				$suscripcion_obj->crear_pre_programacion_venta_continua( $this->id_reloj, $prog_venta['detalle'][0] );
			}else{
				// No debería traer mas de uno, sino definir procedimiento.
				throw new error_negocio("FACTURADOR: Se ha encontrado mas de un articulo para el programa de venta.");
			}
		}
		$this->proceso->agregar_notificacion("Cantidad de pre-programaciones de cuotas continuas generadas:" . count($suscripciones), 'info');
    }

    protected function procesar_ventas_vencidas()
    {
        /**
         *  Iterar todas las ventas en estado VENCIDAS
         *      - generar una nueva venta clonada de la anterior (copiar articulos y convenios)
         *      - en la nueva venta:
         *          - marcar el campo id_venta_origen con una referencia a la vencida.
		 *			- determina la nueva fecha de vencimiento en base a la fecha del proceso y se valor sumarle el valor del parametro "DIAS_1er_V"
         *          - marcar id_periodo_factuacion = $this->id_periodo_facturacion
         *          - Crear un comprobante con dos filas: Importe Adeudado=$$ y Mora=$$
         *          - estado_venta: PREFACTURACION
		 *			- Si param_sistema('APLIC_MORA') ]= 'S
         *				- Calculo de MORA (Segun configuracion):
         *					- "CONSERVAR_MAPEO_ARTICULOS".
         *						Iterar articulos:
         *							- crear un articulo de tipo MORA por cada uno.
         *							- calcular el $ segun el porcentaje establecido para la de mora.
         *							- Al articulo MORA ponerle en el campo id_articulo_copia_mapeo_presupuestario el articulo original
         *					- "MAPEO_MORA"
         *						- crear un unico articulo MORA calculando el $ en base al porcentaje
		 *			 Fin Si
         *      - pasar la venta original al estado ANULADA
         */
		//-- Iterar todas las ventas en estado VENCIDAS de las INSTANCIAS seleccionadas 
		$filtro['id_venta_estado'] = constantes::VENTA_VENCIDA;
		$filtro['ids_servicios_instancias'] = $this->ids_servicios_instancias;
		$ventas_vencidas = sq_uv::catalogo_daos()->ventas()->get_ventas($filtro,null,'ORDER BY v.id','id_venta_origen');
		if(!empty($ventas_vencidas)){
			$ventas_vencidas = $this->filtrar_ventas_que_ya_fueron_revividas_n_veces($ventas_vencidas);
		}
		$this->proceso->agregar_notificacion("Cantidad de ventas vencidas a procesar:" . count($ventas_vencidas), 'info');
		foreach($ventas_vencidas as $venta_vencida)
		{
			// Solo procesar las ventas vencidas que pertenezcan los servicios instancias a facturar
			$servicio_instancia = sq_uv::catalogo_daos()->servicios()->get_servicio_instancia_por_venta($venta_vencida['id']);
			if(!empty($servicio_instancia)) {
				if (!in_array($servicio_instancia['id_servicio_instancia'], $this->ids_servicios_instancias)) {
					// Si el servicio instancia asociado a la venta no se encuentra entre los servicios a facturar saltar a la proxima venta
					continue;
				}
			}else{
				$msg = "Facturador: La venta con el ID. '" . $venta_vencida['id'] . "' no posee una prestacion preprogramada";
				lib::entorno()->log()->error($msg . var_export($servicio_instancia, true));
				throw new error_negocio($msg);
			}
			//-- generar una nueva venta clonada de la anterior (copiar articulos y convenios)
			$venta_clonada = sq_uv::catalogo_modelo()->modelador_venta()->clonar($venta_vencida['id']);
			//-- marcar el campo id_venta_origen con una referencia a la vencida.
			$venta_clonada->set_id_venta_origen($venta_vencida['id']);
			$id_tipo_vencimiento = sq_uv::catalogo_daos()->ventas()->get_tipo_vencimiento($venta_vencida['id']);
			$venta_clonada->set_primer_vencimiento($id_tipo_vencimiento, $this->dias_vencimiento['id_reloj_1er_vencimiento'], $this->dias_vencimiento['dia_1er_vencimiento']);
            if(lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_VENCIMIENTOS) == constantes::SEGUNDO_VENCIMIENTO) {
                $venta_clonada->set_segundo_vencimiento($this->dias_vencimiento['id_reloj_2do_vencimiento'], $this->dias_vencimiento['dia_2do_vencimiento']);
			}
			//-- marcar id_periodo_factuacion = $this->id_periodo_facturacion
			$venta_clonada->set_id_periodo_facturacion( $this->id_reloj );
			
//TODO: -- Crear un comprobante con dos filas: Importe Adeudado=$$ y Mora=$$
//TODO: --  Si param_sistema('APLIC_MORA') ]= 'S
			
			// Iniciar la venta a agrupar en TRUE
			$venta_clonada->set_a_agrupar(TRUE);
			//-- estado_venta: PREFACTURACION
			$venta_clonada->cambiar_estado(constantes::VENTA_ESTADO_PRE_FACT,true);

			//Obtengo la suscripcion para setear el id_medio_pago de la suscripcion en la venta clonada
			$suscripcion = sq_uv::catalogo_daos()->servicios()->obtener_servicios_suscripciones(['id_venta' => $venta_vencida['id']]);
			if(empty($suscripcion)){
				throw new error_negocio("FACTURADOR: No se pudo obtener el servicio suscripcion asociado a la venta vencida: '" . $venta_vencida['id'] . "'");
			}else if(count($suscripcion) > 1){
				throw new error_negocio("FACTURADOR: Se obtuvo mas de una suscripcion asociada a la venta vencida: '" . $venta_vencida['id'] . "'");
			}
			$venta_clonada->set_medio_pago($suscripcion[0]['medio_pago']['id']);
			// Guardar la nueva venta
			$venta_clonada->guardar();
			
			//-- Reasignar el id de la nueva venta a la prestacion pre-programada que estaba asociada la venta origen
			$prestacion_preprog_datos = sq_uv::catalogo_daos()->servicios()->get_prestaciones_programadas(['id_venta' => $venta_vencida['id']]);
			if(empty($prestacion_preprog_datos)){
				throw new error_negocio("FACTURADOR: No se pudo obtener la prestacion pre-programada asociada a la venta vencida: '" . $venta_vencida['id'] . "'");
			}else{
				if(count($prestacion_preprog_datos) > 1){
					throw new error_negocio("FACTURADOR: Se obtuvo mas de una prestacion pre-programada asociada a la venta vencida: '" . $venta_vencida['id'] . "'");
				}
			}
			// Generar cambios para una reversion de facturación
			$this->registrar_punto_reversion($prestacion_preprog_datos[0]['id'], $venta_vencida['id'], constantes::VENTA_VENCIDA);
			
			$prestacion_preprogramada = sq_uv::catalogo_modelo()->prestacion_preprogramada($prestacion_preprog_datos[0]['id']);
			$prestacion_preprogramada->set_id_venta($venta_clonada->get_id());
			$prestacion_preprogramada->guardar();
			
			//-- pasar la venta original al estado ANULADA
			$this->anular_venta($venta_vencida['id']);
		}
    }
	
	private function anular_venta($id)
	{
		$venta_origen = sq_uv::catalogo_modelo()->modelador_venta()->cargar($id);
		$venta_origen->cambiar_estado(constantes::VENTA_ESTADO_ANULADA,true);
		$venta_origen->guardar();
	}
	
	/**
	 * Saca del array que recibe las ventas que ya no debe revivir y las anula
	 * @param array $ventas_vencidas
	 * @return array $ventas_vencidas que si deben ser revividas
	 */
	private function filtrar_ventas_que_ya_fueron_revividas_n_veces(array $ventas_vencidas)
	{
		$ids_ventas_vencidas = array_column($ventas_vencidas, 'id');
		$ids_resultado = [];
		$resultado = [];
		$max_nro_ciclos_suscripcion_activa = (int)lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_CICLOS_REVIVIR_VENTAS_SUSCRIPCION_ACTIVA);
		$max_nro_ciclos_suscripcion_inactiva = (int)lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_CICLOS_REVIVIR_VENTAS_SUSCRIPCION_INACTIVA);
		$dao_ventas = sq_uv::catalogo_daos()->ventas();
		// Recupero los estados de las suscripciones
		$filtro = ['id_venta' => $ids_ventas_vencidas];
		$prestaciones = sq_uv::catalogo_daos()->servicios()->get_prestaciones_programadas($filtro);
		foreach($prestaciones as $p){
			switch ($p['estado']) {//estado, es el estado de la suscripcion
				case constantes::SERVICIOS_SUSCRIPCIONES_ESTADO_ACTIVO:
					if($dao_ventas->get_nro_ciclos_facturador($p['id_venta']) < $max_nro_ciclos_suscripcion_activa){
						$ids_resultado[] = $p['id_venta'];
					}else{
						//anulo las ventas que ya no se deben revivir
						$this->anular_venta($p['id_venta']);
					}
					break;
				case constantes::SERVICIOS_SUSCRIPCIONES_ESTADO_INACTIVO:
					if($dao_ventas->get_nro_ciclos_facturador($p['id_venta']) < $max_nro_ciclos_suscripcion_inactiva){
						$ids_resultado[] = $p['id_venta'];
					}else{
						//anulo las ventas que ya no se deben revivir
						$this->anular_venta($p['id_venta']);
					}
					break;
			}
		}
		
		foreach($ids_resultado as $id){
			foreach($ventas_vencidas as $vv){
				if($id === $vv['id']){
					$resultado[] = $vv;
				}
			}
		}
		return $resultado;
	}
	
    protected function generar_ventas_por_suscripciones()
    {
        /*  Consultar suscripciones activas, trayendo data del servicio y del servicio_instancia
         *  Iterar suscripciones ACTIVAS
         *      1) Iterar prestaciones PREPROGRAMADAS con id_venta == NULL & id_periodo_facturacion = ACTUAL
         *          - Invocar la factory de ventas con los parametros:
         *              - a_agrupar = true,
         *              - id_periodo_facturacion = $this->id_periodo_facturacion
         *              - estado_venta: PREFACTURACION
         *          - Crear articulos programados (registrar en ventas_articulos) y poner en cada fila el ID del servicio que lo genero
         *          - Guardar venta y dejar el ID generado en el registro de la prestacion PREPROGRAMADA que lo genero
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
TODO:!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         *      2) Iterar prestaciones por EVENTO con id_venta == NULL & id_periodo_facturacion = ACTUAL
         *          - Invocar la factory de ventas con los parametros:
         *              - a_agrupar = true,
         *              - id_periodo_facturacion = $this->id_periodo_facturacion
         *              - estado_venta: PREFACTURACION
         *          - Crear articulos programados (registrar en ventas_articulos) y poner en la fila el ID del servicio que lo genero
         *          - Guardar venta y dejar el ID generado en la prestacion por EVENTO que lo genero
         **/
		//-- Iterar suscripciones ACTIVAS
		$filtro['estado'] = constantes::SUSCRIPCION_ESTADO_ACTIVO;
		$filtro['ids_servicios_instancias'] = $this->ids_servicios_instancias;
		$suscripciones = sq_uv::catalogo_daos()->servicios()->get_suscripciones( $filtro );
		$ventas_generadas = [];
		foreach(array_keys($suscripciones) as $id){
			// Cargar la suscripcion
			$suscripcion = sq_uv::catalogo_modelo()->suscripcion( $suscripciones[$id]['id'] );
			//-- Iterar prestaciones PREPROGRAMADAS con id_venta == NULL & id_periodo_facturacion = ACTUAL
			$prestaciones_preprogramadas = $suscripcion->get_prestaciones();
			foreach($prestaciones_preprogramadas as $prestacion){
				if($prestacion->get_id_venta() == NULL AND $prestacion->get_id_reloj_facturador() == $this->id_reloj){
					// obtener el convenio de cofinanciacion si existe
					$id_convenio_de_cofinanciacion = $suscripcion->get_convenio_de_cofinanciacion();
					// si es cofinanciacion
					if( $id_convenio_de_cofinanciacion ){
						$venta = sq_uv::catalogo_modelo()->modelador_venta()->nueva_convenio( $id_convenio_de_cofinanciacion );
					}else{
						$venta = sq_uv::catalogo_modelo()->modelador_venta()->nueva_convenio();
					}
					$venta->set_cliente($suscripciones[$id]['id_cliente']);
					$venta->set_medio_pago($suscripciones[$id]['id_medio_pago']);
					$venta->set_condicion_venta($suscripciones[$id]['id_condicion_venta']);
//TODO: modificar cuando se agregue el campo canal_venta en la suscripcion.
					$venta->set_canal_venta( constantes::CANAL_VENTA_VITUAL );
					$venta->set_leyenda( $prestacion->get_leyenda() );
					
					
					$venta->set_a_agrupar(TRUE);
					$venta->set_id_periodo_facturacion( $this->id_reloj );
					$venta->cambiar_estado( constantes::VENTA_ESTADO_PRE_FACT, true );
					
					// Obtener los articulos de la prestacion preprogramada
					$id_servicio_instancia_prog_venta_articulo = $prestacion->get_id_servicio_instancia_prog_venta_articulo();
					// Inicializar el array de articulos
					$articulos = array();
					// A futuro la consulta deber�a obtener un array de articulos, hoy solo trae uno
					//TODO: CHEQUEAR: la consulta devuelve un registro por valorizacion, es decir que
					//si el servicios_instancias_prog_ventas_articulos tiene 2 valorizaciones
					//(2 registros en servicios_instancias_prog_ventas_articulos_valorizaciones)
					//entonces la consulta retorna 2 filas del mismo articulo, c/u con una valorizacion
					//es decir que si hay mas de una valorizacion el $importe_total de la venta generada calculado mas abajo sera incorrecto
					$articulos[] = sq_uv::catalogo_daos()->servicios()->get_articulo_por_id_serv_prog_venta_art($id_servicio_instancia_prog_venta_articulo);
					if( empty($articulos) ){
						throw new error_negocio("FACTURADOR: GENERAR_VENTAS_POR_SUSCRIPCIONES: No se pudieron obtener los articulos por el programa de venta: " . $suscripcion->get_id_servicio_instancia_prog_venta());
					}
					// obtener los convenios para asociar a los articulos
					$convenios = $suscripcion->get_convenios();
					$importe_total = 0;
					foreach(array_keys($articulos) as $id_a){
						$importe_total += $articulos[$id_a]['importe_unitario'];
						$articulos[$id_a]['cantidad'] = 1;
						// Asociar los convenios menos los de cofinanciacion.
						foreach(array_keys($convenios) as $id_c){
							if( !$convenios[$id_c]['es_cofinanciacion'] and !$articulos[$id_a]['no_aplica_convenio'] ){
								$articulos[$id_a]['convenios'][] = array(	'id' => $convenios[$id_c]['id_convenio']);
							}
						}
					}
					$venta->set_importe_total($importe_total);
					$venta->agregar_articulos($articulos);
					// Setear los dias de vencimientos
					$id_tipo_vencimiento = sq_uv::catalogo_daos()->servicios()->get_tipo_vencimiento_servicio_instancia($suscripciones[$id]['id_servicio_instancia']);
					$venta->set_primer_vencimiento($id_tipo_vencimiento, $this->dias_vencimiento['id_reloj_1er_vencimiento'], $this->dias_vencimiento['dia_1er_vencimiento']);
					if(lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_VENCIMIENTOS) == constantes::SEGUNDO_VENCIMIENTO) {
						$venta->set_segundo_vencimiento($this->dias_vencimiento['id_reloj_2do_vencimiento'], $this->dias_vencimiento['dia_2do_vencimiento']);
					}


					//-- Guardar venta
					$venta->guardar();

                    //-- Obtener el id de la venta que corresponde asociar a la prestación pre-programada
                    if( $id_convenio_de_cofinanciacion ){
                        $id_venta = $venta->get_id_venta_cliente();
                    } else {
                        $id_venta = $venta->get_id();
                    }

					// Generar cambios para una reversion de facturación
					$this->registrar_punto_reversion($prestacion->get_id());
					
					//-- Dejar el ID generado en el registro de la prestacion PREPROGRAMADA que lo genero
					$prestacion->set_id_venta( $id_venta );
					$prestacion->guardar();
					
					$ventas_generadas[] = $venta;
				}
			}
		}
		$this->proceso->agregar_notificacion("Cantidad de ventas generadas en estado de prefacturación:" . count($ventas_generadas), 'info');
		if(!empty($ventas_generadas)){
			//generacion de alarma de facturacion
			$this->evaluar_generacion_de_alarmas($ventas_generadas);
		}
    }
	
	/**
	 * 
	 * @param venta[] $ventas
	 */
	function evaluar_generacion_de_alarmas(array $ventas)
	{
		foreach ($this->ids_facturador_alarmas as $id){
			$facturador_alarma = lib::catalogo_modelo()->facturador_alarma($id);
			foreach ($ventas as $venta){
				$facturador_alarma->evaluar_generacion($this->id_facturacion,$venta);
			}
		}
	}

	protected function procesar_ventas_por_suscripcion_y_cliente()
	{
		/*
         * Hacer una consulta de las ventas del id_periodo_facturacion == ACTUAL con estado en PREFACTURACION,
         *  agrupadas por suscripcion y cliente, con un COUNT de la cantidad de ventas incluidas
         *      - si tiene 1 venta:
         *          - Pasar el flag "a_agrupar" a false.
         *      - si tiene mas de 1 venta:
         *          * Crear una VENTA SUMARIZADA y un comprobante de presentacion asociado
         *              * Iterar las ventas (si la venta es cofinanciada hay que evitarla)
         *              - copiar los articulos y sus convenios
         *                  - si la venta se apoya en una vencida (id_venta_referencia),
         *                      no agregar articulos al comprobante poner frase del estilo "DEUDA ANTERIOR" y total
         *                  - si es una venta basica, agregar los articulos al comprobante
         *          * Aplicar descuentos especiales (EJ: dos postgrados)
         *              - como? -> falta METADATA de que articulos participan de la regla
         *          * Guardar la venta.
         *			Si parametros_sistema('DESC_CRED') = 'S'
         *				* Reclamar credito si existe
         *					- si el saldo del credito del cliente es mayor a cero
         *						- tomar la cantidad disponible hasta el total de la deuda y guardarlo en la tabla de debitos
         *			Fin Si
         */
		//-- Hacer una consulta de las ventas del id_periodo_facturacion == ACTUAL con estado en PREFACTURACION,
		//  agrupadas por cliente, con un COUNT de la cantidad de ventas incluidas
		$filtro['id_venta_estado'] = constantes::VENTA_ESTADO_PRE_FACT;
		$filtro['id_periodo_facturacion'] = $this->id_reloj;
		$filtro['id_tipo_venta'] = constantes::TIPO_VENTA_BASICA;
		$cantidad_ventas_por_suscripcion_cliente = sq_uv::catalogo_daos()->ventas()->get_cantidad_ventas_por_suscripcion_y_cliente($filtro);
		$cantidad_ventas_sumarizadas = 0;
		$ventas_sumarizadas_generadas = [];
		foreach(array_keys($cantidad_ventas_por_suscripcion_cliente) as $id){
			$id_cliente = $cantidad_ventas_por_suscripcion_cliente[$id]['id_cliente'];
			// Cargar las ventas para el cliente actual
			$ventas_por_cliente = sq_uv::catalogo_daos()->ventas()->get_ventas_por_suscripcion_y_cliente($id_cliente, $cantidad_ventas_por_suscripcion_cliente[$id]['id_servicio_suscripcion'], $filtro);
			if($cantidad_ventas_por_suscripcion_cliente[$id]['cantidad'] == 1){
				//-- Pasar el flag "a_agrupar" a false.
				$venta = sq_uv::catalogo_modelo()->modelador_venta()->cargar($ventas_por_cliente[0]['id']);
				$venta->set_a_agrupar(FALSE);
				$venta->guardar();
				
				//-- Si el cliente es beneficiario de convenio al 100% cobrar aplicando el convenio
				if(sq_uv::catalogo_daos()->clientes()->es_beneficiario_convenio_100_y_tiene_pago_unico_asociado($id_cliente)){
					// Aplico el convenio para que la cobranza se genere en cero 
					foreach(array_keys($ventas_por_cliente) as $id_vc){
						$ventas_por_cliente[$id_vc]['importe_total'] = 0;
					}
					$this->generar_cobranza($venta, $ventas_por_cliente, $id_cliente, false, constantes::MEDIO_PAGO_MERCADOPAGO);
				} else {
					//-- Si parametros_sistema('DESC_CRED') = 'S'
					if (lib::parametros_sistema()->get_valor('DESC_CRED') == 'S') {
						//	 Recalamar credito si existe
						$this->reclamar_credito($venta, $ventas_por_cliente, $id_cliente, $venta->get_importe_total());
					}
				}
			}else{
				$leyenda = null;
				$articulos_acumulados = array();
				$ids_ventas = array();
				foreach(array_keys($ventas_por_cliente) as $id_vc){
					// Guardar ids de ventas a sumarizar
					$ids_ventas[] = $ventas_por_cliente[$id_vc]['id'];
					// Cargar venta para obtener articulos y convenios
					$venta_cargada = sq_uv::catalogo_modelo()->modelador_venta()->cargar($ventas_por_cliente[$id_vc]['id']);
					// Acumular los articulos de las diferentes ventas
					$articulos_acumulados = array_merge($articulos_acumulados, $venta_cargada->get_articulos());
					//-- si la venta se apoya en una vencida (id_venta_referencia),
//					if( $venta_cargada->get_id_venta_origen() ){
						// TODO:-- no agregar articulos al comprobante poner frase del estilo "DEUDA ANTERIOR" y total
//					}else{
						// TODO:-- si es una venta basica, agregar los articulos al comprobante
//					}
					$leyenda[] = $venta_cargada->get_leyenda();
				}
				// Obtener el importe total
				$importe_total = 0;
				foreach(array_keys($articulos_acumulados) as $id_a){
					$importe_total += $articulos_acumulados[$id_a]['importe_total'];
				}
				//-- Crear una VENTA SUMARIZADA y un comprobante de presentacion asociado
				$venta = sq_uv::catalogo_modelo()->modelador_venta()->nueva_sumarizada();
				$venta->set_leyenda(  implode(' || ', $leyenda) );
				$venta->set_cliente($id_cliente);
				$venta->set_ids_ventas_sumarizadas($ids_ventas);
				$venta->set_importe_total($importe_total);
				$venta->set_canal_venta( constantes::CANAL_VENTA_FACTURADOR );


                $suscripcion = sq_uv::catalogo_daos()->servicios()->obtener_servicios_suscripciones(['id' => $cantidad_ventas_por_suscripcion_cliente[$id]['id_servicio_suscripcion']]);

                if(empty($suscripcion)){
					throw new error_negocio("FACTURADOR: No se pudo obtener el servicio suscripcion.");
				}else if(count($suscripcion) > 1){
					throw new error_negocio("FACTURADOR: Se obtuvo mas de una suscripcion.");
				}
				$venta->set_medio_pago($suscripcion[0]['medio_pago']['id']);
				$venta->set_condicion_venta( constantes::CONDICION_VENTA_EFECTIVO );
				//-- copiar los articulos y sus convenios
				$venta->agregar_articulos($articulos_acumulados);
//TODO: Aplicar descuentos especiales (EJ: dos postgrados)
				$venta->cambiar_estado( constantes::VENTA_ESTADO_PRE_FACT );
				// Setear el periodo de facturacion actual
				$venta->set_id_periodo_facturacion( $this->id_reloj );
				// Setear los dias de vencimientos
				$venta->set_primer_vencimiento(null, $this->dias_vencimiento['id_reloj_1er_vencimiento'], $this->dias_vencimiento['dia_1er_vencimiento']);
				if(lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_VENCIMIENTOS) == constantes::SEGUNDO_VENCIMIENTO)
				{
					$venta->set_segundo_vencimiento($this->dias_vencimiento['id_reloj_2do_vencimiento'], $this->dias_vencimiento['dia_2do_vencimiento']);
				}

                //-- Guardar la venta.
				$venta->guardar();
					
				//-- Si el cliente es beneficiario de convenio al 100% cobrar aplicando el convenio
				if(sq_uv::catalogo_daos()->clientes()->es_beneficiario_convenio_100_y_tiene_pago_unico_asociado($id_cliente)){
					// Aplico el convenio para que la cobranza se genere en cero 
					foreach(array_keys($ventas_por_cliente) as $id_vc){
						$ventas_por_cliente[$id_vc]['importe_total'] = 0;
					}
					$this->generar_cobranza($venta, $ventas_por_cliente, $id_cliente, false, constantes::MEDIO_PAGO_MERCADOPAGO);
				} else {
					//-- Si parametros_sistema('DESC_CRED') = 'S'
					if( lib::parametros_sistema()->get_valor('DESC_CRED') == 'S'){
						//	 Reclamar credito si existe
						$this->reclamar_credito($venta, $ventas_por_cliente, $id_cliente, $importe_total);
					}
				}
				// Actualizar 'id_venta' en la prestacion pre-programada por el 'id' de la sumarizada
				$filtro_prestacion_pp = [
						'id_servicio_suscripcion'	=> $cantidad_ventas_por_suscripcion_cliente[$id]['id_servicio_suscripcion'],
						'id_reloj_facturador'		=> $this->id_reloj
				];
				// Debería obtener solo una prestación preprogramada
				$datos_prestacion_pp = sq_uv::catalogo_daos()->servicios()->get_prestaciones_programadas($filtro_prestacion_pp);
				if(count($datos_prestacion_pp) > 1){
					$msg = "Se encontraron mas de una prestacion pre-programada para la suscripcion y el reloj.";
					$this->entorno()->log()->err(__METHOD__ . ": " . var_export($filtro_prestacion_pp, true));
					throw new error_negocio("FACTURADOR: " . $msg . " Verifique el log para mas informacion.");
				}
				if(count($datos_prestacion_pp) == 1){
					$prestacion = sq_uv::catalogo_modelo()->prestacion_preprogramada($datos_prestacion_pp[0]['id']);
					$prestacion->set_id_venta( $venta->get_id() );
					$prestacion->guardar();
				}
				if(count($datos_prestacion_pp) == 0){
					// Obtener las prestaciones asociadas a la suscripcion
					$filtro_prestacion_pp = [
						'id_servicio_suscripcion'	=> $cantidad_ventas_por_suscripcion_cliente[$id]['id_servicio_suscripcion'],
						'ventas_no_cobradas'		=> true
					];
					$datos_prestacion_pp = sq_uv::catalogo_daos()->servicios()->get_prestaciones_programadas($filtro_prestacion_pp, "ORDER BY s.id ASC");
					$prestacion = sq_uv::catalogo_modelo()->prestacion_preprogramada(max($datos_prestacion_pp)['id']);
					$prestacion->set_id_venta( $venta->get_id() );
					$prestacion->guardar();
				}

				$cantidad_ventas_sumarizadas++;
				$ventas_sumarizadas_generadas[] = $venta;
			}
		}
		$this->proceso->agregar_notificacion("Cantidad de ventas sumarizadas generadas:" . $cantidad_ventas_sumarizadas, 'info');
		$this->proceso->agregar_notificacion("Cantidad de ventas cobradas:" . $this->cantidad_ventas_cobradas, 'info');
		
		if(!empty($ventas_sumarizadas_generadas)){
			//generacion de alarma de facturacion
			$this->evaluar_generacion_de_alarmas($ventas_sumarizadas_generadas);
		}
	}

	protected function reclamar_credito($venta, $ventas_por_cliente, $id_cliente, $importe_total)
	{
		$cliente = sq_uv::catalogo_modelo()->cliente($id_cliente);
		if(!$cliente->tiene_credito()){
			return;//si el cliente no tiene credito no hago nada
		}
		//-- si el saldo del credito del cliente es mayor a cero
		if($cliente->get_saldo_de_credito() >= $importe_total){
			// Generar cobranza
			$this->generar_cobranza($venta, $ventas_por_cliente, $id_cliente, true, constantes::MEDIO_PAGO_CTA_CTE);
		}else{
			// Pago parcial
		}
	}
	
	protected function generar_cobranza($venta, $ventas_por_cliente, $id_cliente, $utiliza_credito_disponible, $id_medio_pago)
	{
		$datos_cobranza = array();
		$importe_total_cobranza = 0;
		foreach(array_keys($ventas_por_cliente) as $id_vc){
			$datos_cobranza['ventasCobradas'][] = array(
				'id_venta'	=> $ventas_por_cliente[$id_vc]['id'],
				'importe'	=> $ventas_por_cliente[$id_vc]['importe_total']);
			$importe_total_cobranza += $ventas_por_cliente[$id_vc]['importe_total'];
		}
		
		// Generar cobranza
		$datos = [
			'ventasCobradas' => $datos_cobranza['ventasCobradas'],
			'detalles'	=> [
				0 => [
					"concepto" => 'NV',
					"importe"  => $importe_total_cobranza
				]
			],
			'id_cliente' => $id_cliente,
			'importe'	=> $importe_total_cobranza,
			'id_medio_pago' => $id_medio_pago,
			'utiliza_credito_disponible' => $utiliza_credito_disponible
		];
		$transaccion_cobranza = sq_uv::catalogo_transacciones()->cobranza();
		$transaccion_cobranza->set_datos($datos);
		$transaccion_cobranza->procesar();
		//- tomar la cantidad disponible hasta el total de la deuda y guardarlo en la tabla de debitos
		$venta->cambiar_estado( constantes::VENTA_COBRADA );
		$this->cantidad_ventas_cobradas++;
		$this->importe_ventas_cobradas += $importe_total_cobranza;
		//-- Guardar la venta.
		$venta->guardar();
	}

    //-----------------------------------------------------------
    // CONFIRMACION
    //-----------------------------------------------------------

    public function cambiar_estado_ventas($ventas_prefacturadas, $estado_venta)
    {
        /*
		 * ---------------------------------------------------------------------
		*  ---------- TODA ESTA LOGICA SE RESOLVIO EN EL PROCESAMIENTO DE VENTAS
		 * POR CLIENTE ---------------------------------------------------------
        * Cambiar el estado de las ventas de VENTA_ESTADO_PRE_FACT
        *   - si no hay movimiento de debito asociado pasarla a PENDIENTE
        *   - Si hay un movimiento de debito asociado con importe MENOR al total de la venta pasarla a PENDIENTE
        *   - Si hay un movimiento de debito asociado con importe IGUAL al total de la venta pasarla a COBRADA 
         *
         * Atencion, las ventas sumarizadas pasan de estado a las ventas que la componen
        */

		$ids_ventas_agrupadas_en_sumarizada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_sumarizadas(),'id_venta_sumarizada');
        $ids_ventas_tipo_confinanciada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_cofinanciadas(),'id');
		foreach($ventas_prefacturadas as $venta_prefact)
		{
			$venta = sq_uv::catalogo_modelo()->modelador_venta()->cargar($venta_prefact['id']);
			$venta->cambiar_estado( $estado_venta );
			$venta->guardar();
			// Verificar si la venta no es parte de una sumarizada
			if(!in_array($venta_prefact['id'], array_merge($ids_ventas_agrupadas_en_sumarizada, $ids_ventas_tipo_confinanciada))){
				$this->cantidad_ventas_generadas++;
				$this->importe_ventas_generadas += $venta->get_importe_total();
				$this->ventas_generadas[] = $venta_prefact;
			}
		}
		$this->proceso->agregar_notificacion("Cantidad de ventas generadas:" . $this->cantidad_ventas_generadas, 'info');
    }

    protected function notificar_ventas_al_nucleo($ventas_prefacturadas)
    {
        $ids_ventas_agrupadas_en_sumarizada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_sumarizadas(),'id_venta_sumarizada');
        $ids_ventas_tipo_confinanciada = array_column(sq_uv::catalogo_daos()->ventas()->get_ventas_cofinanciadas(),'id');
        foreach($ventas_prefacturadas as $venta_prefact){
            // Verificar si la venta no es parte de una sumarizada o una cofinanciada
            if(!in_array($venta_prefact['id'], array_merge($ids_ventas_agrupadas_en_sumarizada, $ids_ventas_tipo_confinanciada))){
                // Cargar la venta y verificar si se tiene que enviar al nucleo
                $venta = sq_uv::catalogo_modelo()->modelador_venta()->cargar($venta_prefact['id']);
                if($venta->es_notificable_al_nucleo()) {
                    $proceso_enviar_venta = sq_uv::catalogo_procesos()->notificar_venta($venta_prefact['id']);
                    $proceso_enviar_venta->ejecutar();
                }
            }
        }
    }

    protected function cambiar_estado_facturacion_servicios_instancias($estado)
    {
        $facturaciones_servicios_instancias = lib::catalogo_daos()->facturador()->get_facturaciones_servicios_instancias(['estado' => constantes::FACTURADOR_SERVICIO_INSTANCIA_ESTADO_INICIADO]);
        $datos = array(
            'estado' => $estado
        );
        foreach($facturaciones_servicios_instancias as $facturacion_servicio_instancia) {
            $where = [
                'id_servicio_instancia' => $facturacion_servicio_instancia['id_servicio_instancia']
            ];
            $sql_update = sql::sql_array_a_update('facturaciones_servicios_instancias', $datos, $where, $this->entorno()->db());
            $this->entorno()->db()->ejecutar($sql_update);
        }
    }

    //-----------------------------------------------------------
    // REVERTIR
    //-----------------------------------------------------------
	
	protected function revertir_facturacion()
	{
		// Obtener tabla facturaciones_reversiones
		$sql = "SELECT * FROM facturaciones_reversiones;";
		$prestaciones_pp = lib::entorno()->db()->consultar($sql);

		foreach($prestaciones_pp as $prestacion_pp){
			// Actualizar prestaciones pre-programadas
			$prestacion = sq_uv::catalogo_modelo()->prestacion_preprogramada($prestacion_pp['id_servicio_prestacion_pre_programada']);
			$prestacion->set_id_venta( $prestacion_pp['id_venta'] );
			$prestacion->guardar();
			// Actualizar estado de la venta
			if(!is_null($prestacion_pp['id_venta'])){
				$venta = sq_uv::catalogo_modelo()->modelador_venta()->cargar($prestacion_pp['id_venta']);
				$venta->cambiar_estado($prestacion_pp['id_venta_estado'], true);
				$venta->guardar();
			}
		}

		// Eliminar todas las ventas en prefacturacion
		$filtro['id_venta_estado'] = constantes::VENTA_ESTADO_PRE_FACT;
		$filtro['id_periodo_facturacion'] = $this->id_reloj;
		$filtro['excluir_ventas_dependientes_de_sumarizada'] = false;
		$filtro['id_tipo_venta'] = [constantes::TIPO_VENTA_BASICA, constantes::TIPO_VENTA_SUMARIZADA];
		// El orden obtiene primero las sumarizadas (tipo 3) para que no falle por FK
		$order = "ORDER BY id_tipo_venta DESC";
		$ventas = sq_uv::catalogo_daos()->ventas()->get_ventas($filtro, null, $order);
		foreach(array_keys($ventas) as $id){
			$sql = "";
			$sql .= "DELETE FROM ventas_sumarizadas WHERE id_venta_sumariza_a = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM ventas_articulos_convenios WHERE id_venta_articulo IN (SELECT id FROM ventas_articulos WHERE id_venta = " . $ventas[$id]['id'].");";
			$sql .= "DELETE FROM ventas_articulos WHERE id_venta = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM ventas_convenios WHERE id_venta = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM ventas_estados_log WHERE id_venta = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM ventas_vencimientos WHERE id_venta = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM facturaciones_alarmas_generadas_ventas WHERE id_venta = " . $ventas[$id]['id'].";";
			$sql .= "DELETE FROM ventas WHERE id = " . $ventas[$id]['id'].";";
			$this->entorno()->db()->ejecutar($sql);
		}
		//--- Eliminar ventas cofinanciadas
        $filtro['id_venta_estado'] = constantes::VENTA_ESTADO_PRE_FACT;
        $filtro['id_periodo_facturacion'] = $this->id_reloj;
        $filtro['excluir_ventas_dependientes_de_sumarizada'] = false;
        $filtro['id_tipo_venta'] = [constantes::TIPO_VENTA_BASICA, constantes::TIPO_VENTA_COFINANCIADA];
        // El orden obtiene primero las basicas (tipo 1) para que no falle por FK contra las cofianciadas (tipo 2)
        $order = "ORDER BY id_tipo_venta ASC";
        $ventas = sq_uv::catalogo_daos()->ventas()->get_ventas($filtro, null, $order);
        foreach(array_keys($ventas) as $id){
            $sql = "";
            $sql .= "DELETE FROM ventas_sumarizadas WHERE id_venta_sumariza_a = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM ventas_articulos_convenios WHERE id_venta_articulo IN (SELECT id FROM ventas_articulos WHERE id_venta = " . $ventas[$id]['id'].");";
            $sql .= "DELETE FROM ventas_articulos WHERE id_venta = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM ventas_convenios WHERE id_venta = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM ventas_estados_log WHERE id_venta = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM ventas_vencimientos WHERE id_venta = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM facturaciones_alarmas_generadas_ventas WHERE id_venta = " . $ventas[$id]['id'].";";
            $sql .= "DELETE FROM ventas WHERE id = " . $ventas[$id]['id'].";";
            $this->entorno()->db()->ejecutar($sql);
        }
		$this->eliminar_log_reversion();
	}

	function eliminar_log_reversion()
	{
		// Eliminar tabla facturaciones_reversiones
		$sql = "DELETE FROM facturaciones_reversiones;";
		$this->entorno()->db()->ejecutar($sql);
	}
	
	function registrar_punto_reversion($id_sppp, $id_venta = null, $id_venta_estado = null)
	{
		$datos = array(
			'id_facturacion'							=> $this->id_facturacion,
			'id_servicio_prestacion_pre_programada'		=> $id_sppp,
			'id_venta'									=> $id_venta,
			'id_venta_estado'							=> $id_venta_estado
		);
		$sql = sql::sql_array_a_insert('facturaciones_reversiones', $datos, $this->entorno()->db());
		lib::entorno()->db()->ejecutar($sql);
	}
	
	function eliminar_cabecera_facturacion()
	{
		$sql = "DELETE FROM facturaciones WHERE id = " . $this->entorno()->db()->quote($this->id_facturacion) . ";";
		$this->entorno()->db()->ejecutar($sql);
	}
	
	function eliminar_instancias_a_facturar()
	{
		$sql = null;
//		foreach($this->datos['ids_servicios_instancias'] as $instancia){
//			$datos['id_servicio_instancia'] = $instancia;
//			$datos['id_facturacion'] = $this->id_facturacion;
		$sql .= "DELETE FROM facturaciones_servicios_instancias
			 WHERE id_facturacion = " . $this->entorno()->db()->quote($this->id_facturacion) . ";";
//		}
		if($sql) {
			$this->entorno()->db()->ejecutar($sql);
		}else{
			throw new error_negocio("Proceso_revertir_facturacion: No se encontraron servicios instancias para procesar.");
		}
	}
	
	function eliminar_facturaciones_alarmas_configuradas()
	{
		$sql = "DELETE FROM facturaciones_alarmas_configuradas WHERE id_facturacion = " . $this->entorno()->db()->quote($this->id_facturacion) . ";";
		$this->entorno()->db()->ejecutar($sql);
	}
	
	function eliminar_facturaciones_alarmas_generadas()
	{
		$sql = "DELETE FROM facturaciones_alarmas_generadas WHERE id_facturacion = " . $this->entorno()->db()->quote($this->id_facturacion) . ";";
		$this->entorno()->db()->ejecutar($sql);
	}
}
