<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\errores\error_not_found;

class facturador_alarma extends modelo
{
	protected $alarma_generada;
	protected $alarmas_generadas_ventas;
	
	function get_id_tipo_facturador_alarma()
	{
		return $this->datos['id_tipo_facturador_alarma'];
	}	

	function get_par_valor()
	{
		//supongo que siempre hay un solo registro en facturador_alarmas_parametros por registro en facturador_alarmas
		return $this->datos['facturador_alarmas_parametros'][0]['par_valor'];
	}
	
	protected function inicializar()
	{
		$this->datos['id'] = null;
		$this->datos['codigo'] = null;
		$this->datos['descripcion'] = null;
		$this->datos['id_tipo_facturador_alarma'] = null;
		$this->datos['descripcion_comportamiento'] = null;
		$this->datos['estado'] = null;
		$this->datos['estado_actualizado_en'] = null;
		$this->datos['tipo_facturador_alarmas'] = [];
		$this->datos['facturador_alarmas_parametros'] = [];
		$this->alarma_generada = [];
		$this->alarmas_generadas_ventas = [];
	}

	protected function cargar($id)
	{
		$datos = lib::catalogo_daos()->alarmas()->obtener_alarma($id);
		if(is_null($datos)){
			throw new error_not_found("Facturador alarma: La alarma de facturacion que intenta cargar no existe.");
		}else{
			$this->datos = $datos;
		}
	}

	protected function validar()
	{
		// Controlar que esten cargados los datos basicos 
		$campos_obligatorios = [
			'codigo',
			'descripcion',
			'id_tipo_facturador_alarma',
			'estado',
			'estado_actualizado_en'
		];
		$this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __CLASS__);
	}

	function guardar()
	{
		$this->validar();
		if(is_null($this->datos['id'])) {
			$this->grabar();
		} else {
			$this->actualizar();
		}
	}

	protected function actualizar()
	{
		$datos = [
			'codigo'					=> $this->datos['codigo'],
			'descripcion'				=> $this->datos['descripcion'],
			'id_tipo_facturador_alarma' => $this->datos['id_tipo_facturador_alarma'],
			'descripcion_comportamiento'=> $this->datos['descripcion_comportamiento'],
			'estado'					=> $this->datos['estado'],
			'estado_actualizado_en' 	=> 'now()'
		];
		$where = ['id' => $this->datos['id']];
		$sql = sql::sql_array_a_update('facturador_alarmas', $datos, $where, $this->entorno()->db());		
		$this->entorno()->db()->ejecutar($sql);
		$this->actualizar_facturador_alarmas_parametros();
	}
	
	function actualizar_facturador_alarmas_parametros()
	{
		$sql = '';
		foreach($this->datos['facturador_alarmas_parametros'] as $parametros){
			$datos = [
				'par_rango_inferior'=> $parametros['par_rango_inferior'],
				'par_rango_superior'=> $parametros['par_rango_superior'],
				'par_valor'			=> $parametros['par_valor'],
				'par_lista'			=> $parametros['par_lista']
			];
			$where = ['id_facturador_alarma' => $this->datos['id']];
			$sql .= sql::sql_array_a_update('facturador_alarmas_parametros', $datos, $where, $this->entorno()->db());			
		}
		if($sql != ''){
			$this->entorno()->db()->ejecutar($sql);
		}
	}

	protected function grabar()
	{
		$datos = [
			'codigo'					=> $this->datos['codigo'],
			'descripcion'				=> $this->datos['descripcion'],
			'id_tipo_facturador_alarma' => $this->datos['id_tipo_facturador_alarma'],
			'descripcion_comportamiento'=> $this->datos['descripcion_comportamiento'],
			'estado'					=> $this->datos['estado'],
			'estado_actualizado_en' 	=> 'now()'
		];
		$sql = sql::sql_array_a_insert('facturador_alarmas', $datos, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
		$seq = $this->entorno()->db()->consultar_fila("SELECT currval('facturador_alarmas_id_seq') as seq;");
		$this->datos['id'] = $seq['seq'];
		$this->grabar_facturador_alarmas_parametros();
	}
	
	protected function grabar_facturador_alarmas_parametros()
	{
		$sql = '';
		foreach($this->datos['facturador_alarmas_parametros'] as $parametros){
			$datos = [
				'par_rango_inferior'=> $parametros['par_rango_inferior'],
				'par_rango_superior'=> $parametros['par_rango_superior'],
				'par_valor'			=> $parametros['par_valor'],
				'par_lista'			=> $parametros['par_lista']
			];
			$sql .= sql::sql_array_a_insert('facturador_alarmas_parametros', $datos, $this->entorno()->db());
		}
		if($sql != ''){
			$this->entorno()->db()->ejecutar($sql);
		}		
	}
	
	function evaluar_generacion($id_facturacion, $venta)
	{
		switch ($this->get_id_tipo_facturador_alarma()){
			case constantes::ID_TIPO_FACTURADOR_ALARMA_VALOR:
				if($venta->get_importe_total() > $this->get_par_valor()){
					$this->guardar_alarma($id_facturacion,$venta);
				}
				break;
			case constantes::ID_TIPO_FACTURADOR_ALARMA_VENTA_ANULADA:
				// LAS ALARMAS DE ESTE TIPO SE CREAN A MANO EN LA PANTALLA DE FACTURACION
				break;
			// AGREGAR OTROS CASE PARA LOS NUEVOS TIPOS DE ALARMAS DE FACTURACION QUE SE DEFINAN
		}
	}
	
	function guardar_alarma($id_facturacion, $venta,$motivos = null)
	{
		//solamente tengo que crear un registro en facturaciones_alarmas_generadas
		if(empty($this->alarma_generada)){
			$datos_alarma_generada = [				
				'id_facturacion'		=> $id_facturacion,
				'id_facturador_alarma'	=> $this->get_id()
			];
			$sql = sql::sql_array_a_insert('facturaciones_alarmas_generadas', $datos_alarma_generada, $this->entorno()->db());
			$this->entorno()->db()->ejecutar($sql);
			$id = $this->entorno()->db()->ultimo_insert_id("facturaciones_alarmas_generadas_id_seq");
			$this->alarma_generada = [
				'id'					=> $id,
				'id_facturacion'		=> $id_facturacion,
				'id_facturador_alarma'	=> $this->get_id()
			];
		}
		$sql = "INSERT INTO facturaciones_alarmas_generadas_ventas (id_facturacion_alarma_generada, id_venta, motivos) "
				. "VALUES (" . $this->alarma_generada['id'] . ", " . $venta->get_id() . ",'$motivos');";
		$this->entorno()->db()->ejecutar($sql);
	}
}
