<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\procesos\proceso_enviar_emails;
use SIU\SQ\Nucleo\sq_nucleo;

class notificacion extends modelo
{
	protected $tipo_notificacion;
	protected $template_tema;
	protected $template_mensaje;
	protected $dao_notificaciones;
	protected $proceso_envio;

    function __construct($entorno, $id = null, $dao_notificaciones = null)
    {
        $this->entorno = $entorno;
        $this->inicializar();

        if ($dao_notificaciones !== null) {
            $this->dao_notificaciones = $dao_notificaciones;
        }

        if ($id !== null) {
            $this->cargar($id);
        }
    }

    // Permite que los mails online se envien a traves de un proceso externo (el cual se debera ejecutar manualmente por fuera)
    function set_proceso_envio(proceso_enviar_emails $proceso)
    {
        $this->proceso_envio = $proceso;
    }

	function set_id_tipo_notificacion($id_tipo_notificacion)
	{
		$this->datos['id_tipo_notificacion'] = $id_tipo_notificacion;
		$this->tipo_notificacion = $this->dao_notificaciones->get_tipo_notificacion($id_tipo_notificacion);
		$this->template_tema = $this->tipo_notificacion['tema'];
		$this->template_mensaje = $this->tipo_notificacion['mensaje'];
	}
	
	function get_id_cliente()
	{
		return $this->datos['par_id_cliente'];
	}
	
	function get_id_tipo_notificacion()
	{
		return $this->datos['id_tipo_notificacion'];
	}
	
	function get_tema()
	{
		return $this->datos['tema'];
	}
	
	function get_notificado_a()
	{
		return $this->datos['notificado_a'];
	}
	
	function get_mensaje()
	{
		return $this->datos['mensaje'];
	}
	
	function set_id_cliente($id_cliente)
	{
		$this->datos['par_id_cliente'] = $id_cliente;
	}
	
	function set_tema($tema)
	{
		$this->datos['tema'] = $tema;
	}
	
	function set_mensaje($mensaje)
	{
		$this->datos['mensaje'] = $mensaje;
	}
	
	function set_enviada_en($enviada_en)
	{
		$this->datos['enviada_en'] = $enviada_en;
	}
	
	function set_enviada_por($enviada_por)
	{
		$this->datos['enviada_por'] = $enviada_por;
	}
	
	function set_id_servicio_suscripcion($id_servicio_suscripcion)
	{
		$this->datos['par_id_servicio_suscripcion'] = $id_servicio_suscripcion;
	}
	
	function set_id_venta($id_venta)
	{
		$this->datos['par_id_venta'] = $id_venta;
	}
	
	function get_id_venta()
	{
		return $this->datos['par_id_venta'];
	}
	
	function set_id_solicitud_consumo_externo($id_solicitud_consumo_externo)
	{
		$this->datos['par_id_solicitud_consumo_externo'] = $id_solicitud_consumo_externo;
	}
	
	function get_id_solicitud_consumo_externo()
	{
		return $this->datos['par_id_solicitud_consumo_externo'];
	}

	function set_notificado_a($email)
	{
		$this->datos['notificado_a'] = $email;
	}
	
	function set_usuario($usuario)
	{
		$this->datos['par_usuario'] = $usuario;
	}

    function get_dao_notificaciones()
    {
        return $this->dao_notificaciones;
    }

    function set_dao_notificaciones($dao_notificaciones)
    {
        $this->dao_notificaciones = $dao_notificaciones;
    }

    function es_on_line()
	{
		return isset($this->tipo_notificacion) && $this->tipo_notificacion['momento_envio'] == 'O';
	}
	
	protected function inicializar()
	{
		$this->datos['id'] = null;
		$this->datos['id_tipo_notificacion'] = null;
		$this->datos['creada_en'] = 'now()';
		$this->datos['notificado_a'] = null;
		$this->datos['par_id_cliente'] = null;
		$this->datos['par_id_servicio_suscripcion'] = null;
		$this->datos['par_id_venta'] = null;
		$this->datos['par_usuario'] = null;
		$this->datos['par_id_solicitud_consumo_externo'] = null;
		$this->datos['tema'] = null;
		$this->datos['mensaje'] = null;
		$this->datos['enviada_en'] = null;
		$this->datos['enviada_por'] = null;
		$this->tipo_notificacion = null;
		$this->template_tema = null;
		$this->template_mensaje = null;
		$this->dao_notificaciones = lib::catalogo_daos()->notificaciones();
	}
	
	protected function cargar($id)
	{
		$this->datos = $this->dao_notificaciones->get($id);
		$this->tipo_notificacion = $this->datos['tipo_notificacion'];
		$this->template_tema = $this->tipo_notificacion['tema'];
		$this->template_mensaje = $this->tipo_notificacion['mensaje'];
	}
	
	private function cargar_templates()
	{
		$this->tipo_notificacion = $this->dao_notificaciones->get_tipo_notificacion($this->datos['id_tipo_notificacion']);
		$this->template_tema = $this->tipo_notificacion['tema'];
		$this->template_mensaje = $this->tipo_notificacion['mensaje'];
	}

	protected function validar()
	{
		// Controlar que esten cargados los datos basicos
		$campos_obligatorios = [
			'id_tipo_notificacion',
			'creada_en',
			'tema',
			'mensaje'
		];
		
		//agrego los campos obligatorios segun el tipo de notificacion
		if(!is_null($this->tipo_notificacion['parametros'])){
			$campos_obligatorios = array_merge($campos_obligatorios, explode(',', $this->tipo_notificacion['parametros']));
		}
		$this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __CLASS__);
	}

	public function guardar()
	{
		$this->validar();
		if(is_null($this->datos['id'])) {
			$this->grabar_notificacion();
		} else {
			$this->actualizar_notificacion();
		}
		if(is_null($this->datos['enviada_en']) && $this->es_on_line() && !$this->proceso_envio){
			$this->enviar_email();
		}
	}
		
	protected function grabar_notificacion()
	{
        $datos = [
			'id_tipo_notificacion'			=> $this->datos['id_tipo_notificacion'],
			'creada_en'						=> 'now()',
            'notificado_a'					=> $this->datos['notificado_a'],
    		'tema'							=> $this->datos['tema'],
			'mensaje'						=> $this->datos['mensaje'],
			'enviada_en'					=> $this->datos['enviada_en'],
			'enviada_por'					=> $this->datos['enviada_por']
		];
        
        // Agregar parametros obligatorios a los datos
        $campos_obligatorios = explode(',', $this->tipo_notificacion['parametros']);
        foreach($campos_obligatorios as $campo){
   			$datos[ $campo ] = $this->datos[ $campo ];
        }
        
		$sql = sql::sql_array_a_insert('notificaciones', $datos, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
		// Consultar el valor de la secuencia generada
		$this->datos['id'] = $this->entorno()->db()->ultimo_insert_id("notificaciones_id_seq");
	}

	public function eliminar()
	{
		$where = " where id = " . $this->datos['id'];
		$sql = sql::sql_array_tablas_delete(['notificaciones' => $where]);
		$this->entorno()->db()->ejecutar($sql);
	}
	
	protected function actualizar_notificacion()
	{
        $datos = [
			'id_tipo_notificacion'			=> $this->datos['id_tipo_notificacion'],
			'notificado_a'					=> $this->datos['notificado_a'],
			'tema'							=> $this->datos['tema'],
			'mensaje'						=> $this->datos['mensaje'],
			'enviada_en'					=> $this->datos['enviada_en'],
			'enviada_por'					=> $this->datos['enviada_por']
		];
        
        // Agregar parametros obligatorios a los datos
        $campos_obligatorios = explode(',', $this->tipo_notificacion['parametros']);
        foreach($campos_obligatorios as $campo){
   			$datos[ $campo ] = $this->datos[ $campo ];
        }
        
		$where = ['id' => $this->datos['id']];
		$sql = sql::sql_array_a_update('notificaciones', $datos, $where, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
	}
	
	function armar_tema($descripcion)
	{
		if(is_null($this->template_tema)){
			$this->cargar_templates();
		}
		$this->datos['tema'] = str_replace('%1', $descripcion, $this->template_tema);
	}
	
	function armar_mensaje(array $datos_mensaje)
	{
		if(is_null($this->template_mensaje)){
			$this->cargar_templates();
		}
		$mensaje = "";
		//ARMO EL MENSAJE SEGUN EL TIPO DE NOTIFICACION
        $hay_info_personalizada = false;
        if ($this->tipo_notificacion['rol'] == constantes::ROL_CLIENTE) {
            if (isset($this->datos['par_id_venta'])) {
                $info_personalizada = lib::catalogo_daos()->ventas()->get_info_personalizada_por_entidad($this->datos['par_id_venta'], constantes::VENTAS_INFO_PERSONALIZADA_ENTIDAD_MENSAJE_TIPO_NOTIFICACION);
                if ($info_personalizada) {
                    $mensaje = $this->template_mensaje;
                    foreach ($info_personalizada as $info_personalizada_tipo_notificacion) {
                        if ($info_personalizada_tipo_notificacion->codigo == $this->tipo_notificacion['codigo']) {
                            $hay_info_personalizada = true;
                            for ($i = 0; $i < count($info_personalizada_tipo_notificacion->comodines); $i++) {
                                // Armar el mensaje dinamico
                                $mensaje = str_replace('%' . ($i + 1), $info_personalizada_tipo_notificacion->comodines[$i]->valor, $mensaje);
                            }
                        }
                    }
                }
            } else {
                $msg = "No se pudo encontrar el valor 'par_id_venta' para obtener la info personalizada para la notificación.";
                lib::entorno()->log()->warn(__METHOD__  . ": " . $msg);
            }
        }

        if (!$hay_info_personalizada) {
            if ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_MORA ||
                $this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_VENTA ||
                $this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_CUPON_PAGO) {
                $tabla_html = "<table>";
                foreach ($datos_mensaje as $venta) {
                    $tabla_html .= "<tr>";
                    $texto_fila = $this->template_mensaje;
                    if (isset($venta['leyenda'])) {
                        $texto_fila = str_replace('%1', $venta['leyenda'], $texto_fila);
                    }
                    if (isset($venta['cobranza_vence_en'])) {
                        $texto_fila = str_replace('%2', date('d-m-y', strtotime($venta['cobranza_vence_en'])), $texto_fila);
                    }
                    $texto_fila = str_replace('%3', $venta['importe_total'], $texto_fila);
                    $tabla_html .= "<td>$texto_fila</td>";
                    $tabla_html .= "</tr>";
                }
                $tabla_html .= "</table>";
                $mensaje = $tabla_html;
            } elseif ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_REINTEGRO_MERCADO_PAGO ||
                $this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_CREDITO) {
                //si es una devolucion supongo que me va a llegar una unica venta
                $venta = $datos_mensaje[0];
                $mensaje = str_replace('%1', $venta['leyenda'], $this->template_mensaje);
                $mensaje = str_replace('%2', $venta['servicio_version_descripcion'], $mensaje);
            } elseif ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_ERROR_PROCESO) {
                $mensaje = str_replace('%1', $datos_mensaje['descripcion'], $this->template_mensaje);
                $mensaje = str_replace('%2', $datos_mensaje['id_ejecucion'], $mensaje);
                $mensaje = str_replace('%3', date("H:i:s"), $mensaje);
            } elseif ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_TRANSFERENCIA_BANCARIA) {
                $mensaje = str_replace('%1', $datos_mensaje['ventas']['leyenda'], $this->template_mensaje);
                $tabla_html = "<table>";
                foreach ($datos_mensaje['datos_bancarios'] as $cuenta_bancaria) {
                    $tabla_html .= "<tr>";
                    $banco = "<li>Banco: " . $cuenta_bancaria['descripcion'] . ".";
                    $banco .= "<br /> CBU: " . $cuenta_bancaria['cbu'];

                    if ($cuenta_bancaria['alias'])
                        $banco .= "<br /> ALIAS: " . $cuenta_bancaria['alias'];

                    $banco .= "</li> <br />";

                    $tabla_html .= "<td>$banco</td>";
                    $tabla_html .= "</tr>";
                }
                $tabla_html .= "</table>";

                $mensaje = str_replace('%2', $tabla_html, $mensaje);

            } else if ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_AVISO_ALUMNO_POSGRADO || $this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_AVISO_ALUMNO_GRADO) {
                $mensaje = str_replace('%2', $datos_mensaje['descripcion'], $this->template_mensaje);
                $mensaje = str_replace('%3', $datos_mensaje['id_tramite_ref'], $mensaje);
            } else if ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_ENVIO_FACTURA_ELECTRONICA) {
                $mensaje = $this->template_mensaje;
                if (!empty($datos_mensaje['comprobantes'])) {
                    $mensaje = '';
                    foreach ($datos_mensaje['comprobantes'] as $comp) {
                        $mensaje_parcial = str_replace('%1', $comp['venta'], $this->template_mensaje);
                        $mensaje_parcial = str_replace('%2', $comp['numero_comprobante'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%3', $comp['fecha'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%4', $comp['importe'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%5', $comp['cae'], $mensaje_parcial);

                        $breaks = empty($mensaje) ? '' : '<br/><br/>';
                        $mensaje = $mensaje . $breaks . $mensaje_parcial;
                    }
                }
            } else if ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_VENTA_COBRADA_ANULADA) {
                $mensaje = $this->template_mensaje;
                if (!empty($datos_mensaje['comprobantes'])) {
                    $mensaje = '';
                    foreach ($datos_mensaje['comprobantes'] as $comp) {
                        $mensaje_parcial = str_replace('%1', $comp['venta'], $this->template_mensaje);
                        $mensaje_parcial = str_replace('%2', $comp['numero_comprobante'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%3', $comp['fecha_cobro'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%4', $comp['fecha_anulacion'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%5', $comp['nota_credito'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%6', $comp['importe'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%7', $comp['cae'], $mensaje_parcial);
                        $mensaje_parcial = str_replace('%8', $comp['cae_nota'], $mensaje_parcial);
                        // Para evitar multiples br que aparecen dependiendo del mensaje de tipo de notificacion y si hay varias cobranzas
                        $mensaje_parcial = str_replace('<br/><br/>', '<br/>', $mensaje_parcial);

                        $breaks = empty($mensaje) ? '' : '<br/><br/>';
                        $mensaje = $mensaje . $breaks . $mensaje_parcial;
                    }
                }
            } else if ($this->datos['id_tipo_notificacion'] == constantes::TIPO_NOTIFICACION_COMPROBANTE_TRANSFERENCIA_RECHAZADO) {
                $mensaje = str_replace('%1', $datos_mensaje['venta'], $this->template_mensaje);
            } else if (in_array($this->datos['id_tipo_notificacion'], [constantes::TIPO_NOTIFICACION_NUEVA_RECAUDACION, constantes::TIPO_NOTIFICACION_RECAUDACION_ANULADA])) {
                $mensaje = str_replace('%1', $datos_mensaje['id_pilaga'], $this->template_mensaje);
                $mensaje = str_replace('%2', $datos_mensaje['descripcion_especifica_recaudacion'], $mensaje);
            }
        }
		
		$this->datos['mensaje'] = $mensaje;
	}
	
	function enviar_email()
	{
		try{
			$mailer = lib::mailer();
			$email_from = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_SENDER);
			$email_from_name = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_FROM_NAME);
			$mailer->set_from($email_from, $email_from_name);

			$mailer->enviar_email($this->get_tema(),$this->get_mensaje(),$this->get_notificado_a());

			$this->set_enviada_en('now()');
			$this->set_enviada_por($email_from_name);
			$this->guardar();
		} catch (\Exception $ex) {
			$this->entorno()->log()->err("NOTIFICACION: Error al enviar el email on-line: " . $ex->getMessage());
		}
	}

}
