<?php
namespace  SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Uv\constantes;

class reloj extends modelo
{
	protected $id_reloj;
	protected $meses;
	protected $anios;
	protected $id_meses_excepciones;
	protected $datos;
	
	protected function inicializar()
	{
		$this->cargar_meses();
		$this->cargar_anios();
		$this->id_meses_excepciones = array();
	}
	
	
	function __construct($entorno, $id = null, $id_periodo_fact_excepciones = null)
	{
		$this->entorno = $entorno;
		$this->inicializar();
		if($id_periodo_fact_excepciones !== null){
			$this->cargar_periodos_facturacion_excepciones( $id_periodo_fact_excepciones );
		}
		$this->datos = $this->cargar($id);
	}
	
	private function cargar_periodos_facturacion_excepciones( $id_periodo_fact_excepciones )
	{
		$sql = "SELECT id_mes 
				FROM meses_por_periodo_facturacion_excepciones fe 
				WHERE fe.id_periodo_facturacion_excepcion = "
				. $this->entorno()->db()->quote($id_periodo_fact_excepciones) . ";";
		$excepciones = $this->entorno()->db()->consultar($sql);
		foreach($excepciones as $mes){
			$this->id_meses_excepciones[] = $mes['id_mes'];
		}
	}
	
	protected function cargar($id_reloj = null, $filtro = array())
	{
		if($id_reloj !== null){
			$filtro['id'] = $id_reloj;
		}else{
			if( !isset($this->id_reloj) ){
				$filtro['id_reloj_facturador_estado'] = constantes::RELOJ_FACTURACION_ESTADO_ACTUAL;
			}
		}
		$datos = lib::catalogo_daos()->reloj_faturador()->obtener_reloj($filtro);
		if($datos){
			$this->id_reloj = $datos['id'];
		}else{
			$error = implode(', ', array_map(function ($v, $k) { return $k . '=' . $v; }, $filtro, array_keys($filtro)));
			throw new error_negocio("RELOJ: No se pudo cargar el reloj con el filtro: " . $error);
		}
		return $datos;
	}
	
	function get_descripcion_anio()
	{
		foreach($this->anios as $anio){
			if($anio['id'] == $this->datos['id_anio']){
				return $anio['codigo'];
			}
		}
	}
	
	function get_descripcion_mes()
	{
		foreach($this->meses as $mes){
			if($mes['id'] == $this->datos['id_mes']){
				return $mes['codigo'];
			}
		}
	}
	
	function get_actual() 
	{
		return $this->id_reloj;
	}
	
	// El valor 'posicion' es cero o numero positivo. Si es cero devuelve el 
	// ID del reloj seteado en el constructor
	function get_reloj( $posicion = 0, $id_frecuencia = null )
	{
		// Obtengo el anio y mes del reloj ingresado al constructor
		$id_mes = $this->datos['id_mes'];
		$id_anio = $this->datos['id_anio'];
		// Obtener la cantidad de meses que representa el $id_frecuencia
		if($id_frecuencia){
			$frecuencia = $this->get_frecuencia_facturacion($id_frecuencia);
		}else{
			$frecuencia = null;
		}
		// Si hay posicion seteada busco el id del mes
		if($posicion != 0){
			for($p = 0 ; $p < $posicion ; $p++){
				// Si la frecuencia esta seteada afecta los meses
				if($frecuencia){
					for($i = 0 ; $i < $frecuencia['cantidad_meses'] ; $i++){
						// Sumar un mes al mes y año del reloj actual.
						$datos = $this->get_proximo_mes_y_anio($id_mes, $id_anio);
						$id_mes = $datos['id_mes'];
						$id_anio = $datos['id_anio'];
					}
				}else{
					$datos = $this->get_proximo_mes_y_anio($id_mes, $id_anio);
					$id_mes = $datos['id_mes'];
					$id_anio = $datos['id_anio'];
				}
				$datos = $this->cargar(null, array('id_mes' => $id_mes, 'id_anio' => $id_anio));
			}
			$id = $datos['id'];
		}else{
			$id = $this->id_reloj;
		}
		return $id;
	}
	
	/*
	 * Actualiza el estado del reloj actual a EJECUTADO 
	 * Obtiene el proximo reloj y lo deja como ACTUAL
	 */
	
	function avanzar()
	{
		//validar que existen 24 relojes posteriores al actual
		//y si no existen crear los que hagan falta.
		if (!$this->existen_relojes_posteriores()) {
			$this->crear_relojes_faltantes();
		}
		
		// Actualizar el reloj
		$where = array("id" => $this->id_reloj);
		$datos = array("id_reloj_facturador_estado" => constantes::RELOJ_FACTURACION_ESTADO_EJECUTADO);
		$sql = sql::sql_array_a_update("reloj_facturador", $datos, $where, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
		
		$where = array("id" => $this->get_reloj(1));
		$datos = array("id_reloj_facturador_estado" => constantes::RELOJ_FACTURACION_ESTADO_ACTUAL);
		$sql = sql::sql_array_a_update("reloj_facturador", $datos, $where, $this->entorno()->db());
		$this->entorno()->db()->ejecutar($sql);
	}
	
	/**
	 * Valida que existan los relojes de facturación necesarios para alcanzar 
	 * el buffer deseado específicado en los parámetros de sistema.
	 */
	function existen_relojes_posteriores()
	{
		$cant_relojes_necesarios = lib::parametros_sistema()->get_valor(constantes::BUFFER_RELOJES);
		$cant_relojes_posteriores = lib::catalogo_daos()->reloj_faturador()
				->obtener_cantidad_de_relojes_posteriores_al_actual();
		return $cant_relojes_posteriores >= $cant_relojes_necesarios;
	}
	
	/**
	 * Genera los relojes de facturación necesarios para alcanzar 
	 * el buffer deseado específicado en los parámetros de sistema.
	 */
	function crear_relojes_faltantes()
	{
		$cant_relojes_necesarios = lib::parametros_sistema()->get_valor(constantes::BUFFER_RELOJES);
		$relojes_faltantes = $cant_relojes_necesarios - lib::catalogo_daos()->reloj_faturador()
				->obtener_cantidad_de_relojes_posteriores_al_actual();
		
		$ultimo_reloj = lib::catalogo_daos()->reloj_faturador()
				->obtener_ultimo_reloj();
		
		$ultimo = [
			'mes' => $ultimo_reloj['numero_mes'],
			'anio' => $ultimo_reloj['numero_anio'],
		];
		
		for ($i = 0; $i < ($relojes_faltantes); $i++) {
			//si es el mes 12 tengo que comenzar a insertar los relojes para el siguiente año
			if ($ultimo["mes"] == 12) {
				$ultimo_anio = $ultimo['anio'] + 1;
				$anio = lib::catalogo_daos()->reloj_faturador()->obtener_anio(['numero_anio' => $ultimo_anio]);
				if(is_null($anio)) { // si no existe el año siguiente lo inserto en la base
					$datos_anio = [
						'codigo' => $ultimo_anio,
						'descripcion' => 'Año siguiente al ' . $ultimo['anio'],
						'anio' => $ultimo_anio
					];
					$sql = sql::sql_array_a_insert('anios', $datos_anio, $this->entorno()->db());
					$this->entorno()->db()->ejecutar($sql);
					$anio = lib::catalogo_daos()->reloj_faturador()->obtener_anio(['numero_anio' => $ultimo_anio]);
				}
				$mes = lib::catalogo_daos()->reloj_faturador()->obtener_mes(['numero_mes' => 1]); //obtengo el primer mes del año
			} else {
				$anio = lib::catalogo_daos()->reloj_faturador()->obtener_anio(['numero_anio' => $ultimo['anio']]);
				$mes = lib::catalogo_daos()->reloj_faturador()->obtener_mes(['numero_mes' => $ultimo['mes'] + 1]); //obtengo el mes siguiente al último generado
			}

			//insert row reloj facturador
			$datos_reloj = [
				'codigo' => substr($mes['descripcion'], 0, 3) . "." . $anio['anio'],
				'descripcion' => $mes['descripcion'] . " " . $anio['anio'],
				'id_anio' => $anio['id'],
				'id_mes' => $mes['id'],
				'dia_inicio_facturacion' => 1,
				'dia_limite_facturacion' => lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_FACTURACION_LIMITE),
				'id_reloj_facturador_estado' => constantes::RELOJ_FACTURACION_ESTADO_NOINICIADO,
				'creado_por' => constantes::USUARIO_INTERNO_SQ_UV,
				'creado_en' => 'now()'
			];
			$sql = sql::sql_array_a_insert('reloj_facturador', $datos_reloj, $this->entorno()->db());
			$this->entorno()->db()->ejecutar($sql);
			
			$ultimo['anio'] = $anio['anio'];
			$ultimo['mes'] = $mes['numero_mes'];
		}
	}

	public function crear_reloj($anio, $mes)
    {
        $rs_anio = lib::catalogo_daos()->reloj_faturador()->obtener_anio(['numero_anio' => $anio]);
        if(is_null($rs_anio)) { // si no existe el año siguiente lo inserto en la base
            $datos_anio = [
                'codigo' => $anio,
                'descripcion' => $anio,
                'anio' => $anio
            ];
            $sql = sql::sql_array_a_insert('anios', $datos_anio, $this->entorno()->db());
            $this->entorno()->db()->ejecutar($sql);
            $rs_anio = lib::catalogo_daos()->reloj_faturador()->obtener_anio(['numero_anio' => $anio]);
        }
        $rs_mes = lib::catalogo_daos()->reloj_faturador()->obtener_mes(['numero_mes' => $mes]);

        // Crear reloj
        $datos_reloj = [
            'codigo' => substr($rs_mes['descripcion'], 0, 3) . "." . $rs_anio['anio'],
            'descripcion' => $rs_mes['descripcion'] . " " . $rs_anio['anio'],
            'id_anio' => $rs_anio['id'],
            'id_mes' => $rs_mes['id'],
            'dia_inicio_facturacion' => 1,
            'dia_limite_facturacion' => lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_FACTURACION_LIMITE),
            'id_reloj_facturador_estado' => constantes::RELOJ_FACTURACION_ESTADO_NOINICIADO,
            'creado_por' => constantes::USUARIO_INTERNO_SQ_UV,
            'creado_en' => 'now()'
        ];
        $sql = sql::sql_array_a_insert('reloj_facturador', $datos_reloj, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
        return $this->entorno()->db()->ultimo_insert_id("reloj_facturador_id_seq");
    }
	
	//-------------------- MESES Y ANIOS  --------------------------------------
	
	private function cargar_meses()
	{
		$sql = "SELECT id, numero_mes, codigo FROM meses ORDER BY numero_mes ASC;";
		$this->meses = $this->entorno()->db()->consultar($sql);
	}
	
	private function cargar_anios()
	{
		$sql = "SELECT id, anio, codigo FROM anios ORDER BY anio ASC;";
		$this->anios = $this->entorno()->db()->consultar($sql);
	}
	
	private function get_proximo_mes_y_anio($id_mes, $id_anio)
	{
		foreach(array_keys($this->meses) as $idm){
			if($id_mes == $this->meses[$idm]['id']){
				if($this->meses[$idm]['numero_mes'] == '12'){
					// Reinicio el mes 
					foreach($this->meses as $mes){
						if($mes['numero_mes'] == 1){
							$salida['id_mes'] = $mes['id'];
						}
					}
					// Sumo un anio
					foreach(array_keys($this->anios) as $ida){
						if($id_anio == $this->anios[$ida]['id']){
							if(isset($this->anios[$ida + 1])){
								$salida['id_anio'] = $this->anios[$ida + 1]['id'];
							}else{
								throw new error_negocio(
									"RELOJ: No se pudo determinar el proximo año.");
							}
						}
					}
				}else{
					// Sumo un mes y sigo en el mismo anio
					if(isset($this->meses[$idm + 1])){
						$salida['id_mes'] = $this->meses[$idm + 1]['id'];
						$salida['id_anio'] = $id_anio;
					}else{
						throw new error_negocio(
								"RELOJ: No se pudo determinar el proximo mes.");
					}
				}
			}
		}
		// Evaluar si el mes obtenido se encuentra dentro de las excepciones
		if( in_array($salida['id_mes'], $this->id_meses_excepciones) ){
			return $this->get_proximo_mes_y_anio($salida['id_mes'], $salida['id_anio']);
		}
		
		return $salida;
	}
	
	function get_frecuencia_facturacion($id_frecuencia)
	{
		$sql = "SELECT cantidad_meses FROM frecuencias_facturacion
				WHERE id = " . $this->entorno()->db()->quote($id_frecuencia) . ";";
		return $this->entorno()->db()->consultar_fila($sql);
	}
	
	function get_numero_mes()
	{
		foreach($this->meses as $mes){
			if($mes['id'] == $this->datos['id_mes']){
				return $mes['numero_mes'];
			}
		}
	}
	
	function get_numero_anio()
	{
		foreach($this->anios as $anio){
			if($anio['id'] == $this->datos['id_anio']){
				return $anio['anio'];
			}
		}
	}
	
	protected function validar(){}
	public function guardar(){}
}
