<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\modelo\modelo;
use SIU\SQ\Lib\errores\error_not_found;

class venta_info_personalizada extends modelo
{
    protected function inicializar()
    {
        $this->datos['id'] = null;
        $this->datos['id_venta'] = null;
        $this->datos['datos'] = null;
    }
    
    protected function cargar($id_venta)
    {
        $this->datos = lib::catalogo_daos()->ventas()->get_info_personalizada($id_venta);
        if (empty($this->datos)) {
            throw new error_not_found(__METHOD__ . " No se pudo recuperar la información personalizada correspondiente al ID de venta $id_venta");
        }
    }
    
    protected function validar()
    {
        $campos_obligatorios = ['id_venta','datos'];
        $this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __METHOD__);
    }
    
    public function guardar()
    {
        $this->validar();
        if (! $this->get_id()) {
            $this->grabar();
        } else {
            // No existe ningun caso que requiera su uso
            //$this->actualizar();
        }
        return $this->get_id();
    }
    
    private function grabar()
    {
        $datos = [
            'id_venta'	=> $this->datos['id_venta'],
            'datos'		=> json_encode($this->datos['datos'], JSON_UNESCAPED_UNICODE)
        ];

        if (json_last_error() != JSON_ERROR_NONE) {
            $msg = "Error al consumir el JSON del campo 'datos'";
            lib::entorno()->log()->error(__METHOD__ . ": Error json (" . json_last_error() . ")" . $msg);
            throw new error_negocio($msg);
        }

        $sql = sql::sql_array_a_insert('ventas_info_personalizada', $datos, $this->entorno()->db());
    
        $this->entorno()->db()->ejecutar($sql);
        $this->datos['id'] = $this->entorno()->db()->ultimo_insert_id("ventas_info_personalizada_id_seq");
    }
    
    private function actualizar()
    {
        $datos = [
            'datos'	=> $this->datos['datos']
        ];
        
        $where = ['id' => $this->get_id()];
        $sql = sql::sql_array_a_update('ventas_info_personalizada', $datos, $where, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
    }
    
    //--SETTERS
    public function set_id_venta($id)
    {
        $this->datos['id_venta'] = $id;
    }

    public function set_datos($datos)
    {
        $this->datos['datos'] = $datos;
    }
}
