<?php
namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\entorno\configurador;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\lib;

abstract class proceso 
{	
	protected $id_proceso;
	protected $estado_ejecucion_proceso;
	private $entorno;
	protected $id_ejecucion;
	protected $total_a_procesar;
	protected $procesados_ok;
	protected $procesados_error;
	protected $estado_inicial;
	protected $verboso;

	const ESTADO_INDEFINIDO	= 0;
	const ESTADO_OK			= 1;
	const ESTADO_ERROR		= 2;
	const ESTADO_EN_ESPERA	= 3;
	const ESTADO_ACTIVO		= 4;
	const ESTADO_INACTIVO	= 5;
	
	function __construct($id_proceso, $entorno, $id_proceso_ejecucion = null, $estado_inicial = null) 
	{
		$this->inicializar($estado_inicial);
		$this->id_proceso = $id_proceso;
		$this->entorno = $entorno;
		$this->id_ejecucion = $id_proceso_ejecucion;
		$this->cargar();
	}
	
	protected function inicializar($estado_inicial)
	{
		$this->id_ejecucion 	= 0;
		$this->total_a_procesar	= null;
		$this->procesados_ok	= 0;
		$this->procesados_error	= 0;
		$this->estado_inicial 	= ($estado_inicial) ?  $estado_inicial : self::ESTADO_INDEFINIDO;
		$this->verboso			= false;
	}

	private function cargar()
	{
		$sql = "SELECT
					id,
					codigo,
					descripcion,
					id_tipo_proceso,
					hora,
					prioridad,
					estado,
					estado_actualizado_en
				FROM
					procesos
				WHERE
					id = " . $this->entorno->db()->quote($this->id_proceso)
				. "	;";
		$this->datos = $this->entorno->db()->consultar_fila($sql);
		
		if( !is_null($this->id_ejecucion) ){
			$sql = "SELECT
						total_a_procesar,
						procesados_ok,
						procesados_error,
						id_proceso_ejecucion_estado
					FROM
						procesos_ejecuciones
					WHERE
						id = " . $this->entorno->db()->quote($this->id_ejecucion) 
				.	";";
			$proceso_ejecucion = $this->entorno->db()->consultar_fila($sql);
			if(!empty($proceso_ejecucion)){
				$this->total_a_procesar 		= $proceso_ejecucion['total_a_procesar'];
				$this->procesados_ok			= $proceso_ejecucion['procesados_ok'];
				$this->procesados_error			= $proceso_ejecucion['procesados_error'];
				$this->estado_ejecucion_proceso	= $proceso_ejecucion['id_proceso_ejecucion_estado'];
			}else{
				throw new error("PROCESO: Se intenta cargar una ejecuci�n de proceso pero no se encontraron datos relacionados.");
			}
		}
	}
	
	function ejecutar()
	{
		if( !lib::es_modo_mantenimiento() ) {
			$this->id_ejecucion = $this->iniciar_registro_ejecucion();
			$this->entorno()->log()->info("INICIANDO PROCESO: [" . $this->datos['codigo'] . "] " . $this->datos['descripcion']);
			$this->ejecutar_especifico();
			$this->entorno()->log()->info("FINALIZANDO PROCESO: [" . $this->datos['codigo'] . "] " . $this->datos['descripcion']);
		}else{
			$config = new configurador();
			$dir = $config->get_config_dir();
			$proyecto = parse_ini_file($dir . "/../toba/proyecto.ini", TRUE);
			$mensaje = $proyecto['proyecto']['mantenimiento_mensaje'] . PHP_EOL;
			echo $mensaje; 
			$this->entorno->log()->warn($mensaje . " Intentando ejecutar el proceso: " 
				. "([" . $this->datos['codigo'] . "] " . $this->datos['descripcion'] . ")");
		}
	}
	
	abstract function ejecutar_especifico();
	
	function entorno()
	{
		if(!isset($this->entorno)){
			throw new error("PROCESO - El entorno no esta cargado");
		}
		return $this->entorno;		
	}

	public function get_id_proceso()
	{
		return $this->id_proceso;
	}
	
	function set_entorno($entorno)
	{
		$this->entorno = $entorno;
	}
	
	function get_id_ejecucion()
	{
		return $this->id_ejecucion;
	}
	
	function get_estado()
	{
		return $this->estado_ejecucion_proceso;
	}
	
	function set_estado($estado)
	{
		$this->estado_ejecucion_proceso = $estado;
	}
	
	function set_total_a_procesar($total)
	{
		$this->total_a_procesar = $total;
	}
	
	function agregar_procesado_ok()
	{
		$this->procesados_ok++;
	}
	
	function agregar_procesado_error()
	{
		$this->procesados_error++;
	}

	function es_verboso()
	{
		return $this->verboso;
	}

	function set_verboso($verboso)
	{
		$this->verboso = $verboso;
	}
	
	function iniciar_registro_ejecucion()
	{
		// Regitrar en la tabla 'procesos_ejecuciones'
		// INSERT procesos ejecuiones con id_proceso y ESTADO 0
		// 0, indefinido, 1 ok, 2 ERROR (tabla: procesos_estados)
		$datos = array( 'id_proceso'					=> $this->id_proceso,
						'total_a_procesar'				=> 0,
						'procesados_ok'					=> 0,
						'procesados_error'				=> 0,
						'iniciado_en'					=> 'now()',
						'finalizado_en'					=> NULL,
						'id_proceso_ejecucion_estado'	=> $this->estado_inicial);
//		$datos = null;
		$sql = sql::sql_array_a_insert('procesos_ejecuciones', $datos, $this->entorno->db());  // Se le pas� la DB porque internamente necesita la funci�n de quoteo de la clase db... 
		$this->entorno->db()->ejecutar($sql);
		
		// Consultar el valor de la secuencia generada en la  tabla 'procesos_ejecuciones'
		$seq = $this->entorno->db()->consultar_fila("SELECT currval('procesos_ejecuciones_id_seq') as seq;");
		return $seq['seq'];
	}
	
	function finalizar_registro_ejecucion($estado=null)
	{
		$estado_final = isset($estado)? $estado : $this->estado_ejecucion_proceso;
		$datos = [
			'total_a_procesar'				=> $this->total_a_procesar,
			'finalizado_en'					=> 'now()',
			'id_proceso_ejecucion_estado'	=> $estado_final
		];
		$where = array( 'id' => $this->id_ejecucion, 'id_proceso' => $this->id_proceso);
		$sql = sql::sql_array_a_update('procesos_ejecuciones', $datos, $where, $this->entorno->db());
		$this->entorno->db()->ejecutar($sql);
		if($estado_final == proceso::ESTADO_ERROR){
			$this->notificar_error();
		}
	}
	
	function agregar_notificacion($mensaje, $severidad)
	{
		try {
			//------ Definir que se deber�a guardar en este campo...
			$tmp = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
			unset($tmp[1]['file']);
			unset($tmp[1]['function']);
			unset($tmp[1]['type']);
			$componente = json_encode($tmp[1]);
			// Cuando es invocado desde la pantalla de toba la conexion a la db es LATIN1
			// por esa razón necesito convertir el mensaje en esa codificación.
			if ($this->es_encoding_latin1()) {
				$mensaje_utf8 = mb_convert_encoding($mensaje, 'ISO-8859-1', 'UTF-8');
			} else {
				$mensaje_utf8 = $mensaje;
			}

			//---------------------------------------------

			$datos = array('id_proceso_ejecucion' => $this->id_ejecucion,
				'ocurrido_en' => 'now()',
				'severidad' => $severidad,
				'componente' => $componente,
				'texto' => $mensaje_utf8);
			$sql = sql::sql_array_a_insert('procesos_ejecuciones_log', $datos, $this->entorno->db());  // Se le pas� la DB porque internamente necesita la funci�n de quoteo de la clase db...
			$this->entorno->db()->ejecutar($sql);
		} catch (\Exception $ex) {
			$msg = "Error al grabar la tabla 'procesos_ejecuciones_log'. Id_proceso: " . $this->id_proceso . " - SQL: " . $sql;
			$this->entorno->log()->error(__METHOD__ . ": " . $msg . " - " . $ex->getMessage());
			throw new error($msg);
		}
	}
	
	function get_notificaciones()
	{
		return lib::catalogo_daos()->procesos()->get_procesos_ejecuciones_log($this->id_ejecucion);
	}
	
	protected function es_encoding_latin1()
	{
		$db = $this->entorno->db();
		preg_match("/Client\sEncoding:\s(LATIN1)+;/", $db->get_pdo()->getAttribute(\PDO::ATTR_SERVER_INFO), $matches);
		if(empty($matches)){
			return false;
		}else {
			return true;
		}
	}

	public function actualizar_registros_procesados()
	{
		$datos = [
			'procesados_ok'	=> $this->procesados_ok,
			'procesados_error' => $this->procesados_error
		];
		// Consultar
		if($this->estado_ejecucion_proceso == self::ESTADO_EN_ESPERA){
			if($this->total_a_procesar == $this->procesados_ok){
				$datos['id_proceso_ejecucion_estado'] = self::ESTADO_OK;
			} else{
				if($this->total_a_procesar == ($this->procesados_ok + $this->procesados_error)){
					$datos['id_proceso_ejecucion_estado'] = self::ESTADO_ERROR;
				}
			}
		}
		$where = array( 'id' => $this->id_ejecucion, 'id_proceso' => $this->id_proceso);
		$sql = sql::sql_array_a_update('procesos_ejecuciones', $datos, $where, $this->entorno->db());
		$this->entorno->db()->ejecutar($sql);
	}
	
	/**
	 * Guarda un registro en notificaciones_clientes para que luego la levante el proceso de envio de emails
	 */
    public function notificar_error()
    {
        $crear_notificacion = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_NOTIFICACION_ERROR_PROCESO);
        if($crear_notificacion == 'S'){
            $param_usu_admin = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_USUARIO_ADMIN);
            $param_email_admin = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_ADMIN);
            if(empty($param_usu_admin) || empty($param_email_admin)){
                $this->agregar_notificacion('El nombre o el email del usuario administrador no esta seteado en los parametros del sistema. '
                    . 'Para hacerlo puede usar el comando crear_usuario_admin', 'error');
            }else{
                $notificacion = lib::catalogo_modelo()->notificacion();
                $notificacion->set_usuario($param_usu_admin);
                $notificacion->set_notificado_a($param_email_admin);
                $notificacion->set_id_tipo_notificacion(constantes::TIPO_NOTIFICACION_ERROR_PROCESO);
                $notificacion->armar_tema($this->datos['descripcion']);
                $this->datos['id_ejecucion'] = $this->id_ejecucion;
                $notificacion->armar_mensaje($this->datos);
                $notificacion->guardar();
            }
        }
	}
}

