<?php
namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_db;
use SIU\SQ\Lib\errores\error_negocio;

class proceso_confirmar_facturacion extends proceso
{
	protected $cantidad_ventas_generadas;
	protected $importe_ventas_generadas;
	protected $ventas_generadas;
	
	function ejecutar_especifico()
	{
		try {
			
			$facturador = lib::catalogo_modelo()->facturador();
			$facturador->set_proceso($this);
			$facturador->confirmar();

			$this->cantidad_ventas_generadas	= $facturador->get_cantidad_ventas_generadas();
			$this->importe_ventas_generadas		= $facturador->get_importe_ventas_generadas();
			$this->ventas_generadas				= $facturador->get_ventas_generadas();
			
			$this->agregar_notificacion("Ha finalizado el proceso de confirmación de la facturación", 'info');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_OK);
		} catch (error_db $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error_negocio $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (\Exception $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		}
	}

	public function get_cantidad_ventas_generadas()
	{
		return $this->cantidad_ventas_generadas;
	}

	public function get_importe_ventas_generadas()
	{
		return $this->importe_ventas_generadas;
	}

	public function get_ventas_generadas()
	{
		return $this->ventas_generadas;
	}
}
