<?php

namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_rest;
use SIU\SQ\Lib\procesos\proceso;
use SIU\SQ\Lib\modelo\notificacion;
use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\entorno;
use SIU\SQ\Lib\util\Documentos;
use SIU\SQ\Lib\entorno\configurador;
use setasign\Fpdi;

class proceso_enviar_emails extends proceso
{
	protected $ids_envios_notificaciones_fallados = array();
	protected $ids_envios_notificaciones_exitosos = array();
	protected $email_from_defecto = null;//esta variable la usa el mailer
	protected $email_from_name = null;
	protected $notificaciones_a_enviar = [];
	protected $comprobantes_creados_ids = [];


	protected $mensajes_de_error = "";
	
	function get_ids_envios_notificaciones_fallados()
	{
		return $this->ids_envios_notificaciones_fallados;
	}

	function get_ids_envios_notificaciones_exitosos()
	{
		return $this->ids_envios_notificaciones_exitosos;
	}
	
	function set_notificaciones_a_enviar(array $notificaciones)
	{
		$this->notificaciones_a_enviar = $notificaciones;
	}

	function agregar_notificacion_a_enviar(notificacion $notif)
    {
        $this->notificaciones_a_enviar[] = $notif;
    }
		
	public function ejecutar_especifico()
	{
		$this->set_estado(self::ESTADO_OK);
		try{
			$this->procesar_envio_mail();
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->actualizar_registros_procesados();
			$this->finalizar_registro_ejecucion(self::ESTADO_ERROR);
			throw $e;
		}
		$this->actualizar_registros_procesados();
		$this->finalizar_registro_ejecucion();
	}

	protected function procesar_envio_mail()
	{
		if(empty($this->notificaciones_a_enviar)){
			$filtro = ['enviada_en__null' => true, 'notificado_a__not_null' => true];
			$this->notificaciones_a_enviar = lib::catalogo_daos()->notificaciones()->get_lista($filtro);
		}
		$mailer = lib::mailer();
		$this->email_from_defecto = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_SENDER);
		$this->email_from_name = lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CODIGO_EMAIL_FROM_NAME);
		$mailer->set_from($this->email_from_defecto, $this->email_from_name);

		$this->set_total_a_procesar(count($this->notificaciones_a_enviar ?: []));
		foreach($this->notificaciones_a_enviar as $n){
			try{
				if(is_array($n)){
					$n = lib::catalogo_modelo()->notificacion($n['id']);
					// Por si la notificacion online fallo al enviarse y el proceso de consola la levanta.
					$n->set_proceso_envio($this);
				}
				$this->enviar_email($mailer,$n);
				$this->ids_envios_notificaciones_exitosos[] = $n->get_id();
				$this->agregar_procesado_ok();
			} catch (\Exception $ex) {
				$this->ids_envios_notificaciones_fallados[] = $n->get_id();
				$this->agregar_procesado_error();
				$this->mensajes_de_error .= $ex->getMessage() . PHP_EOL;
				$this->entorno()->log()->err($ex->getMessage());
				$this->agregar_notificacion($ex->getMessage(), 'error');
				$this->set_estado(self::ESTADO_ERROR);
			}
		}

		$this->log_info();
	}
	
	// Procesa la notificacion (por ejemplo para agregar comprobantes en nucleo/academico)
	protected function procesar_tipo_notificacion($mailer, $notificacion, &$comprobantes)
    {
        if($notificacion->get_id_tipo_notificacion() == constantes::TIPO_NOTIFICACION_CUPON_PAGO){
            $venta = lib::catalogo_daos()->notificaciones()->get_lista(['id' => $notificacion->get_id()])[0];
            if(isset($venta['id_comprobante'])){
                $comprobante = lib::catalogo_daos()->comprobantes()->get($venta['id_comprobante']);
                $partes = explode('/', $comprobante['id_rdi']);
                $mailer->adjuntar_archivo($comprobante['id_rdi'], end($partes),  'base64', 'application/pdf');
                $comprobantes[] = ['id' => $comprobante['id']];
            }
        }
    }

	function enviar_email($mailer,notificacion $notificacion)
	{
        // Limpiar adjuntos pre nuevo envio
        $mailer->eliminar_adjuntos();
		//parametros para mailer
		$destinatario_email = $notificacion->get_notificado_a();
		if(empty($destinatario_email)){
			throw new error(__METHOD__ . ' No se pudo obtener el email destinatario de la notificacion con ID: ' . $notificacion->get_id());
		}
		$asunto = $notificacion->get_tema();
		$cuerpo_email = $notificacion->get_mensaje();
        $this->entorno()->log()->info(__METHOD__ . ": Se está procesando la notificación con ID: " . $notificacion->get_id());

        // COmprobantes generados para la notificacion
        $comprobantes_generados = array();
        $this->procesar_tipo_notificacion($mailer, $notificacion, $comprobantes_generados);

		$mailer->enviar_email($asunto,$cuerpo_email,$destinatario_email);
        $mailer->eliminar_adjuntos();

        $this->entorno()->log()->info(__METHOD__ . ": Comprobantes temporales a borrar: " . print_r($comprobantes_generados, true));
        if (!empty($comprobantes_generados)) {
            // Eliminiar los PDF's generados
            foreach ($comprobantes_generados as $comp_gen) {
                if (isset($comp_gen['path'])) {
                    $id = isset($comp_gen['id']) ? $comp_gen['id'] : 'FUSIONADO';
                    $this->entorno()->log()->debug(__METHOD__ . ": Borrando comprobante temporal: ID=" . $id. '   PATH=' . $comp_gen['path']);
                    unlink($comp_gen['path']);
                } else {
                    $this->entorno()->log()->warn(__METHOD__ . ": No se encontro path de un comprobante generado en el envio de mail: " . print_r($comp_gen, true));
                }
            }
            // Guardar los ids de comprobantes generados para el log
            foreach ($comprobantes_generados as $comp_gen) {
                if (isset($comp_gen['id'])) {
                    $this->comprobantes_creados_ids[] = $comp_gen['id'];
                }
            }
        }

		$notificacion->set_enviada_en('now()');
		$notificacion->set_enviada_por($this->email_from_name);
		$notificacion->guardar();
	}
    
    public function generar_pdf(array $datos_venta, $nombre_comprobante_jasper, $datos_qr = null)
    {
        $generador_de_pdf = lib::toba_vista_jasperreports_json();
        $generador_de_pdf->set_datos_venta($datos_venta);
        $generador_de_pdf->set_path_reporte(\toba::proyecto()->get_path().'/../reportes/' . $nombre_comprobante_jasper);
        if ($datos_qr) {
            $generador_de_pdf->set_datos_qr($datos_qr);
        }
        $generador_de_pdf->generar_pdf();
        $archivo_generado = $generador_de_pdf->get_nombre_archivo_generado();

        if (lib::parametros_sistema()->get_valor("USARAIDOCU") == "S") {
            $this->exportar_a_arai_documentos($datos_venta, $archivo_generado);
        }

        return $archivo_generado;
    }
    
    public function generar_documento_unico($comprobantes_generados)
    {
        // https://manuals.setasign.com/fpdi-manual/v2/the-fpdi-class/examples/
        $pdf = new Fpdi\Fpdi();
        
        foreach ($comprobantes_generados as $file) {
            if (is_array($file)) {
                $file = $file['path'];
            }
            $pageCount = $pdf->setSourceFile($file);
            for ($pageNo = 1; $pageNo <= $pageCount; $pageNo++) {
                $templateId = $pdf->importPage($pageNo);
                $size = $pdf->getTemplateSize($templateId);
                $pdf->AddPage($size['orientation'], $size);
                $pdf->useTemplate($templateId);
            }
        }
        
        $archivo_salida = \toba::proyecto()->get_path_temp() . '/' . date('YmdHis') . '_factura' . '.pdf';
        
        $pdf->Output('F', $archivo_salida);
        
        return $archivo_salida;
    }

	protected function log_info()
	{
		$log_info = count($this->ids_envios_notificaciones_exitosos) . " notificaciones enviadas por email; " . PHP_EOL;
		if(count($this->ids_envios_notificaciones_exitosos) > 0){
			$log_info .= "Los ids de la tabla notificaciones son: " . implode(',',$this->ids_envios_notificaciones_exitosos) . PHP_EOL;
			$log_info .= "Se generaron los comprobantes id: " . implode(',', $this->comprobantes_creados_ids) . PHP_EOL;
		}
		$log_info .= '--------------------------------------------------------------------' . PHP_EOL;
		$log_info .= "Envio de emails fallados: " . count($this->ids_envios_notificaciones_fallados) . PHP_EOL;
		if(count($this->ids_envios_notificaciones_fallados) > 0){
			$log_info .= "Los ids de la tabla notificaciones que fallaron son: " . implode(',', $this->ids_envios_notificaciones_fallados) . PHP_EOL;
			$log_info .= '--------------------------------------------------------------------' . PHP_EOL;
			$log_info .= "Errores: " . PHP_EOL . $this->mensajes_de_error . PHP_EOL;
		}
		
		//feedback en consola
		if($this->es_verboso()){
			echo $log_info;
		}
		$this->agregar_notificacion($log_info, 'info');
	}

    protected function recuperar_de_arai_documentos($datos)
    {
        $this->entorno()->log()->debug('Se recupera el comprobante de Arai Documentos');

        $archivo = Documentos\AraiDocumentos::downloadDocumento($datos['uuid_arai_documentos']);
        $nombre_archivo = \toba::proyecto()->get_path_temp() . '/' . $datos['uuid_arai_documentos']  . '_ad' . '.pdf';

        if (file_put_contents($nombre_archivo, $archivo) === false) {
            throw new error(__METHOD__ . ' Error al crear el archivo recibido por Arai Documentos.');
        } else {
            return realpath($nombre_archivo);
        }
    }

    protected function exportar_a_arai_documentos($comprobante, $path_to_file)
    {
        $this->entorno()->log()->debug('Se envia el comprobante a Arai Documentos');

        $param_config = new configurador();

        try {
            $data = [];
            $data['archivo'] = base64_encode(file_get_contents($path_to_file));
            $data['autorizacion']['usuario'] = $param_config->sq_ini['AraiDocumentos']['USUARIO_ARAI'];
            $data['origen']['id_usuario_sso'] = $param_config->sq_ini['AraiDocumentos']['USUARIO_SSO'];
            $data['origen']['cuenta_usuario'] = $param_config->sq_ini['AraiDocumentos']['USUARIO_CUENTA'];
            $data['documento']['tipo'] = $param_config->sq_ini['AraiDocumentos']['TIPO_DOCUMENTO'];
            $data['documento']['titulo'] = 'Factura: ' . $comprobante['numero_comprobante'] ;
            $data['documento']['descripcion'] = 'Cliente: ' . $comprobante['cliente']['descripcion'];
            $data['documento']['id_tramite'] = $comprobante['id_comprobante'];
            $data['documento']['referencia_interna'] = $comprobante['id_comprobante'];
            $data['documento']['identificador'] = $comprobante['id_comprobante'];
            $data['documento']['atributos'] = '';

            $araiDoc = Documentos\AraiDocumentos::exportarDocumento($data);

            $comp = lib::catalogo_modelo()->comprobante($comprobante['id_comprobante']);
            $comp->set_uuid_arai_documentos($araiDoc['uid_documento']);
            $comp->set_presentacion(json_decode($comp->get_presentacion()));
            $comp->guardar();
        } catch (\Exception $e) {
            $this->entorno()->log()->err($e->getMessage() . "\n" . $e->getTraceAsString());
            throw new error_rest($e->getMessage(), $e->getCode());
        }
    }

}
