<?php
namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_db;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\lib;

class proceso_generar_comprobantes extends proceso
{
	protected $ventas;
	
	function ejecutar_especifico()
	{
		try {
			// Iniciar el proceso en estado OK (al procesarse puede pasar a ERROR)
			$this->set_estado(self::ESTADO_OK);
			
			$this->cargar_configuraciones();
			
			$this->validar();
			
			$this->generar_comprobantes();
			
			//feedback en consola
			if($this->es_verboso()){
				foreach($this->get_notificaciones() as $notificacion) {
					echo $notificacion['texto'] . PHP_EOL;
				}
			}
			$this->finalizar_registro_ejecucion();
		} catch (error_db $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error_negocio $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (\Exception $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		}
	}
	
	protected function validar()
	{
		$this->validar_instalacion_jasper();
	}
	
	protected function validar_instalacion_jasper()
	{
		lib::toba_vista_jasperreports_json();
	}
	
	protected function cargar_configuraciones()
	{
		// Sin configuraciones por defecto
	}
	
	function generar_pdf(array $datos_venta, $nombre_comprobante_jasper)
	{
		$generador_de_pdf = lib::toba_vista_jasperreports_json();
		$generador_de_pdf->set_datos_venta($datos_venta);
		$generador_de_pdf->set_path_reporte(\toba::proyecto()->get_path().'/../reportes/' . $nombre_comprobante_jasper);
		$generador_de_pdf->generar_pdf();
		return $generador_de_pdf->get_nombre_archivo_generado();
	}
	
	public function set_ventas($ventas)
	{
		$this->ventas = $ventas;
	}
	
	public function generar_comprobantes()
	{
	}
}