<?php
namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_db;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\lib;
use SIU\SQ\Uv\constantes;

class proceso_generar_facturacion extends proceso
{
	protected $id_reloj_1er_vencimiento;
	protected $dia_1er_vencimiento;
	protected $id_reloj_2do_vencimiento;
	protected $dia_2do_vencimiento;
	protected $ids_servicios_instancias;
	protected $ids_facturador_alarmas;
	protected $id_facturacion;
	protected $cantidad_ventas_pre_generadas;
	protected $importe_ventas_pre_generadas;
	protected $cantidad_ventas_cobradas;
	protected $importe_ventas_cobradas;
	
	function ejecutar_especifico()
	{
		try {
			
			$facturador = lib::catalogo_modelo()->facturador();
			$facturador->set_proceso($this);
			$facturador->set_dia_primer_vencimiento($this->id_reloj_1er_vencimiento, $this->dia_1er_vencimiento);
			if(lib::parametros_sistema()->get_valor(constantes::PARAMETROS_SISTEMA_CANTIDAD_VENCIMIENTOS) == constantes::SEGUNDO_VENCIMIENTO) {
				$facturador->set_dia_segundo_vencimiento($this->id_reloj_2do_vencimiento, $this->dia_2do_vencimiento);
			}
			$facturador->set_instancias_a_facturar($this->ids_servicios_instancias);
			if(isset($this->ids_facturador_alarmas)){
				$facturador->set_ids_facturador_alarmas($this->ids_facturador_alarmas);
			}
			$facturador->generar();

			$this->id_facturacion					= $facturador->get_id_facturacion();
			$this->cantidad_ventas_pre_generadas	= $facturador->get_cantidad_ventas_pre_generadas();
			$this->importe_ventas_pre_generadas		= $facturador->get_importe_ventas_pre_generadas();
			$this->cantidad_ventas_cobradas			= $facturador->get_cantidad_ventas_cobradas();
			$this->importe_ventas_cobradas			= $facturador->get_importe_ventas_cobradas();

			$this->agregar_notificacion("Ha finalizado el proceso de generación de la facturación", 'info');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_OK);
		} catch (error_db $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error_negocio $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (\Exception $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		}
	}

	public function set_dia_primer_vencimiento($id_reloj_1er_vencimiento, $dia_1er_vencimiento)
	{
		$this->id_reloj_1er_vencimiento = $id_reloj_1er_vencimiento;
		$this->dia_1er_vencimiento = $dia_1er_vencimiento;
	}
	
	public function set_dia_segundo_vencimiento($id_reloj_2do_vencimiento, $dia_2do_vencimiento)
	{
		$this->id_reloj_2do_vencimiento = $id_reloj_2do_vencimiento;
		$this->dia_2do_vencimiento = $dia_2do_vencimiento;
	}

	public function set_parametros_alarmas($parametros)
	{

	}

	public function set_instancias_a_facturar($id_servicios_instancias)
	{
		$this->ids_servicios_instancias = $id_servicios_instancias;
	}

	public function get_id_facturacion()
	{
		return $this->id_facturacion;
	}

	public function get_ids_facturador_alarmas()
	{
		return $this->ids_facturador_alarmas;
	}

	public function set_ids_facturador_alarmas(array $ids_alarmas)
	{
		$this->ids_facturador_alarmas = $ids_alarmas;
	}

	public function get_cantidad_ventas_cobradas()
	{
		return $this->cantidad_ventas_cobradas;
	}

	public function get_importe_ventas_cobradas()
	{
		return $this->importe_ventas_cobradas;
	}

	public function get_cantidad_ventas_pre_generadas()
	{
		return $this->cantidad_ventas_pre_generadas;
	}

	public function get_importe_ventas_pre_generadas()
	{
		return $this->importe_ventas_pre_generadas;
	}
}
