<?php

namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\entorno\configurador;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\util\calculador;
use SIU\SQ\Lib\util\instanciador_afip;

/**
 * Toma las ventas en estado COBRADA sin comprobantes y genera los comprobantes llamando a los web services de AFIP.
 * Hace inserts en tabla comprobantes y updates en ventas
 * Se deben obtener los certificados en AFIP
 * Para obtener certificados de AFIP para llamar a los WS http://www.afip.gob.ar/ws/
 *
 * REQUERIMIENTOS: SOAP
$apt-get update -y && apt-get install -y libxml2-dev php-soap && apt-get clean -y && docker-php-ext-install soap
 *
 * SINCRONIZAR RELOJ CON SERVIDOR NTP
instalar ntp y ntpdate
$sudo apt-get install ntp
$sudo apt-get install ntpdate

sincronizar con servidor NTP UNQ
$sudo ntpdate 10.22.0.9

o agregar linea
server 10.22.0.9
en etc/ntp.conf y reiniciar servidor
$etc/init.d/ntp restart
 *
 */

class proceso_generar_facturas_afip extends proceso
{
	protected $cobranzas;
	protected $afip;
	protected $transaccion_comprobante_actualizar_numeracion_externa;
    protected $parametrizacion;
    protected $datos_codigo_barras = [];

	public function set_transaccion_comprobante_actualizar_numeracion_externa($transaccion_comprobante_actualizar_numeracion_externa)
	{
		$this->transaccion_comprobante_actualizar_numeracion_externa = $transaccion_comprobante_actualizar_numeracion_externa;
	}

	public function set_cobranzas($cobranzas)
	{
		$this->cobranzas = $cobranzas;
	}

	function ejecutar_especifico()
	{
		try {
			// Iniciar el proceso en estado OK (al procesarse puede pasar a ERROR)
			$this->set_estado(self::ESTADO_OK);

			$this->cargar_configuraciones();

            $this->cargar_parametrizaciones();

            $this->validar();

			$this->generar_facturas_afip();

			//feedback en consola
			if($this->es_verboso()){
				foreach($this->get_notificaciones() as $notificacion) {
					echo $notificacion['texto'] . PHP_EOL;
				}
			}
			$this->finalizar_registro_ejecucion();
		} catch (error_db $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error_negocio $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (error $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		} catch (\Exception $e) {
			$this->agregar_notificacion($e->getMessage(), 'error');
			$this->finalizar_registro_ejecucion(proceso::ESTADO_ERROR);
			throw $e;
		}
	}

	//---- CARGAR CONFIGURACIONES ---------------

	function cargar_configuraciones()
	{
        $afip = new instanciador_afip($this->entorno());
        $this->afip = $afip->getAfip();

        $configurador = new configurador();
		$this->datos_codigo_barras['cuit'] = $configurador->sq_ini['factura_afip']['cuit'];
	}

    function cargar_parametrizaciones()
    {
        $this->parametrizacion = lib::catalogo_daos()->comprobantes()->get_parametrizaciones();

        if (!$this->parametrizacion['comprobante_tipo_factura']) {
            throw new error_negocio(__METHOD__ . ': No se encuentra definido el parametro comprobante tipo factura');
        }
    }

    //---- VALIDACIONES -------------


	function validar()
	{
		// Validar que se haya inyectado la transacción del comprobante
		if(!is_object($this->transaccion_comprobante_actualizar_numeracion_externa)){
			throw new error(__METHOD__ . ": Falta asignar una transaccion de comprobantes.");
		}
		// Validar punto de cobranza seteado en las cobranzas
		$cobranzas_sin_pcob = array();
		foreach($this->cobranzas as $cobranza) {
			if(empty($cobranza['codigo_punto_cobranza'])) {
				$cobranzas_sin_pcob[] = $cobranza['id'];
			}
		}
		if(!empty($cobranzas_sin_pcob)){
			throw new error_negocio(__METHOD__ . ": No se encontraron puntos de cobranza asociados a las siguientes Id's de cobranzas: " . implode(",", $cobranzas_sin_pcob));
		}
	}

	//----- GENERAR FACTURA AFIP -----------------------

	function generar_facturas_afip()
	{

		$this->agregar_notificacion("Cantidad de cobranzas a procesar: " . count($this->cobranzas), 'info');

		// Generar CAE's, PDF's y grabar comprobantes
		$total_errores_afip = 0;
		$comprobantes = [];
		
		foreach ($this->cobranzas as $cobranza) {
			try{
				// Obtener el CAE mediante WS de AFIP
				$CAE = $this->generar_CAE($cobranza);

                $this->datos_codigo_barras['punto_venta'] = $cobranza['codigo_punto_cobranza'];
                $this->datos_codigo_barras['CAE'] = $CAE['CAE'];
                $this->datos_codigo_barras['CAEFchVto'] = $CAE['CAEFchVto'];

				$cobranza['numero_comprobante'] = $CAE['voucher_number'];
				$cobranza['CAE'] = $CAE['CAE'];
				$cobranza['CAEFchVto'] = $CAE['CAEFchVto'];
                $cobranza['codigo_barras'] = $this->generar_codigo_barras();

				$this->agregar_notificacion("Cobranza ID: " .$cobranza['id'] . " Num.Comprobante: " . $cobranza['numero_comprobante'] . " CAE: " . $cobranza['CAE'], 'info');

				try{
					$this->actualizar_comprobantes($cobranza);
					$this->agregar_notificacion("Comprobante ID: " .$cobranza['id_comprobante'] . " actualizado.", 'info');

					// BUscar toda la info de los comprobantes y engancharlos a sus respectivas ventas
					$datos = $this->generar_datos_notificacion($cobranza);
					if (isset($datos) && isset($datos['id_venta'])) {
						$comprobantes[$datos['id_venta']][] = $datos;
					}
				} catch (\Exception $ex) {
					$this->set_estado(self::ESTADO_ERROR);
					$msg = "Error al generar comprobante de la cobranza ID: " . $cobranza['id'] . ". Mensaje: " . $ex->getMessage();
					$this->agregar_notificacion($msg, 'error');
				}

			} catch (\Exception $ex) {
                $cobranza['error_CAE'] = $ex->getMessage();
                $this->actualizar_comprobantes($cobranza);

				$this->set_estado(self::ESTADO_ERROR);
				$total_errores_afip++;
				$msg = "Error al obtener CAE de la cobranza ID: " . $cobranza['id'] . ". Mensaje: " . $ex->getMessage();
				$this->agregar_notificacion($msg, 'error');
			}
		}

		if(count($this->cobranzas)) {
			$this->agregar_notificacion("Cantidad de errores al generar CAE: " . $total_errores_afip, 'info');
		}
		
        if (!empty($comprobantes)) {
            // Por cada venta enviar notificacion.
            foreach ($comprobantes as $id_venta => $datos_comp) {
                //Una venta puede tener mas de una cobranza, por lo tanto se crea una notificacion por venta
                //verifico si existe una notificacion creada para dicha venta, de lo contrario la creo
                $notificacion = lib::catalogo_daos()->notificaciones()->get_lista(['id_venta' => $id_venta]);
                if (empty($notificacion) && !empty($datos_comp)) {
                    $this->crear_notificacion_cliente($id_venta, $datos_comp[0]['id_cliente'], $datos_comp);
                }
            }
        }
	}

    // Genera los datos que se van a utilizar para crear la notificacion mas adelante
    function generar_datos_notificacion($cobranza) {
        return null;
    }

    // Crea notificaciones segun la informacion devuelta por el metodo generar_datos_notificacion
    function crear_notificacion_cliente($id_venta, $id_cliente, $datos_notificacion) {}

	function generar_CAE($cobranza)
	{
		$factura_electronica = new \SIU\Afip\WebService\FacturaElectronica($this->afip);

		$cliente = $this->determinar_cliente($cobranza);

        $info_personalizada = $this->get_datos_venta_info_personalizada($cobranza);

		$data = [
			'CantReg'   	=> 1,  // Cantidad de comprobantes a registrar
			'PtoVta'    	=> $cobranza['codigo_punto_cobranza'],  // Punto de venta
            'CbteTipo'  	=> $this->parametrizacion['comprobante_tipo_factura'],  // Tipo de comprobante (ver tipos disponibles)
			'Concepto'  	=> $info_personalizada['Concepto'],  // Concepto del Comprobante: (1)Productos, (2)Servicios, (3)Productos y Servicios
			'DocTipo'   	=> $cliente['tipo_documento'], //96, // Tipo de documento del comprador (99 consumidor final, ver tipos disponibles)
			'DocNro'    	=> $cliente['numero_documento'], //$cobranza['cliente']['numero_documento'],  // Número de documento del comprador (0 consumidor final)
			'CbteFch'   	=> intval(date('Ymd')), // (Opcional) Fecha del comprobante (yyyymmdd) o fecha actual si es nulo
			'ImpTotal'  	=> $cobranza['importe_total'], // Importe total del comprobante
			'ImpTotConc'    => 0,   // Importe neto no gravado
			'ImpNeto'   	=> $cobranza['importe_total'], // Importe neto gravado
			'ImpOpEx'   	=> 0,   // Importe exento de IVA
			'ImpIVA'    	=> 0,  //Importe total de IVA
			'ImpTrib'   	=> 0,   //Importe total de tributos
			'MonId'     	=> 'PES', //Tipo de moneda usada en el comprobante (ver tipos disponibles)('PES' para pesos argentinos)
			'MonCotiz'  	=> 1     // Cotización de la moneda usada (1 para pesos argentinos)
		];

		// Si el concepto es servicio o producto y servicio se debe enviar las fechas de
		if( in_array($info_personalizada['Concepto'], [constantes::AFIP_TIPO_CONCEPTO_SERVICIO, constantes::AFIP_TIPO_CONCEPTO_PRODUCTO_SERVICIO])) {
            $data_servicio = [
                'FchServDesde' => $info_personalizada['FchServDesde'],
                'FchServHasta' => $info_personalizada['FchServHasta'],
                'FchVtoPago' => intval(date('Ymd'))
            ];
            $data = array_merge($data, $data_servicio);
        }

		return $factura_electronica->crearProximoComprobante($data);
	}

	protected function get_datos_venta_info_personalizada($cobranza)
    {
        // Generar los valores por defecto
        $salida['Concepto'] = constantes::AFIP_TIPO_CONCEPTO_SERVICIO;
        $salida['FchServDesde'] = intval(date('Ym01', strtotime($cobranza['cobrada_en'])));
        $salida['FchServHasta'] = intval(date('Ymt', strtotime($cobranza['cobrada_en'])));

        // Buscar valores personalizados
        if(isset($cobranza['ventas_info_personalizada']) && !is_null($cobranza['ventas_info_personalizada'])){
            $ventas_info_personalizada = json_decode($cobranza['ventas_info_personalizada'], true);
            foreach($ventas_info_personalizada as $info_personalizada){
                if(isset($info_personalizada['entidad']) AND $info_personalizada['entidad'] == constantes::VENTAS_INFO_PERSONALIZADA_ENTIDAD_AFIP) {
                    if(isset($info_personalizada['datos']['concepto'])) {
                        // Validar la estructura
                        if(isset($info_personalizada['datos']['concepto']['codigo']) && in_array($info_personalizada['datos']['concepto']['codigo'], [1,2,3])){
                            $salida['Concepto'] = $info_personalizada['datos']['concepto']['codigo'];
                        } else {
                            $msg = "Se encontraron errores en el concepto definido como: " . var_export($info_personalizada['datos']['concepto'], true);
                            lib::entorno()->log()->error(__METHOD__ . ": " . $msg);
                            throw new error_negocio($msg);
                        }
                    }
                    if(isset($info_personalizada['datos']['periodo_facturado'])) {
                        // Validar la estructura
                        if(isset($info_personalizada['datos']['periodo_facturado']['fecha_desde']) && isset($info_personalizada['datos']['periodo_facturado']['fecha_hasta'])) {
                            $salida['FchServDesde'] = intval(date('Ymd', strtotime($info_personalizada['datos']['periodo_facturado']['fecha_desde'])));
                            $salida['FchServHasta'] = intval(date('Ymd', strtotime($info_personalizada['datos']['periodo_facturado']['fecha_hasta'])));
                        }
                    }
                }
            }
        }

        return $salida;
    }

    function determinar_cliente($cobranza) {
        $cliente = [];

        if ($cobranza['responsable_pago']['es_persona_fisica'] == true) {
            $cliente['numero_documento'] = $cobranza['responsable_pago']['numero_documento'];
        } else {
            $cliente['numero_documento'] = $cobranza['responsable_pago']['identificacion_fiscal'];
        }

        if (!isset($cobranza['responsable_pago']['tipo_documento_afip']) or empty($cobranza['responsable_pago']['tipo_documento_afip'])){
            $msg = "No se pudo procesar el comprobante para el cliente con el número de documento: " . $cobranza['responsable_pago']['numero_documento'] . ". No se encuentra definido en SQ el tipo de documento AFIP correspondiente al tipo de documento: '" . $cobranza['responsable_pago']['tipo_documento'] . "'.";
            lib::entorno()->log()->error(__METHOD__ . ': ' . $msg  );
            throw new error_negocio($msg );
        }
        $cliente['tipo_documento'] = $cobranza['responsable_pago']['tipo_documento_afip'];

        return $cliente;
    }


	/**
	 *
	 * @param string $CAE
	 * @param array $datos_cobranza
	 * @return int
	 */
	function actualizar_comprobantes($datos_cobranza)
	{
	    if(isset($datos_cobranza['error_CAE'])) {
            $datos = [
                'id' => $datos_cobranza['id_comprobante'],
                'error_CAE' => $datos_cobranza['error_CAE']
            ];
        } else {
            $datos = array(
                'id' => $datos_cobranza['id_comprobante'],
                'numero_comprobante' => $datos_cobranza['numero_comprobante'],
                'CAE' => $datos_cobranza['CAE'],
                'CAEFchVto' => $datos_cobranza['CAEFchVto'],
                'codigo_barras' => $this->datos_codigo_barras['codigo']
            );
        }

		$this->transaccion_comprobante_actualizar_numeracion_externa->set_datos($datos);
		$this->transaccion_comprobante_actualizar_numeracion_externa->procesar();
	}

	function generar_codigo_barras()
    {
        $this->datos_codigo_barras['codigo'] = calculador::digito_verificador_cae(
            $this->datos_codigo_barras['cuit'],
            $this->parametrizacion['comprobante_tipo_factura'],
            str_pad($this->datos_codigo_barras['punto_venta'], 4, 0, STR_PAD_LEFT),
            $this->datos_codigo_barras['CAE'],
            date("Ymd", strtotime($this->datos_codigo_barras['CAEFchVto']))
        );
    }
}
