<?php

namespace SIU\SQ\Lib\sistemas_externos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\constantes;

class paypertic extends gestor_pago
{
    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function anular_cupon($id_referencia)
    {
        try {
            $r = lib::entorno()->get_cliente('sq_pagos_backend')->put('paypertic/expirar_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function devolver_pago($id_referencia)
    {
        try {
            lib::entorno()->get_cliente('sq_pagos_backend')->put('paypertic/devolver_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia
     * @throws error
     */
    public function expirar_url_de_pago($id_referencia)
    {
        try {
            $merchant_order = $this->obtener_merchant_order_mp($id_referencia);
            $id_preferencia = $merchant_order['preference_id'];
            lib::entorno()->get_cliente('sq_pagos_backend')->put('paypertic/expirar_preferencia/' . $id_preferencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_preferencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_preferencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia 
     * @return mixed
     * @throws error
     */
    public function obtener_datos_del_pago($id_referencia)
    {
        try {
            $response = lib::entorno()->get_cliente('sq_pagos_backend')->get('paypertic/payment/' . $id_referencia);
            $datos = json_decode($response->getBody()->getContents(), true);
            return $datos;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @throws error
     */
    public function get_metodos_pagos()
    {
        
    }
}
