<?php
namespace SIU\SQ\Lib;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_db;
use SIU\SQ\Lib\errores\error_negocio;

abstract class transaccion
{
	protected $entorno;
	protected $datos;
	protected $id;
	
	function __construct($entorno, $id = null)
	{
		$this->entorno = $entorno;
		$this->id = $id;
	}

	/**
	 * @return entorno
	 */
	function entorno()
	{
		if(!isset($this->entorno)){
			throw new error("PROCESO - El entorno no esta cargado");
		}
		return $this->entorno;		
	}
	
	// Metodos obligatorios
	abstract protected function procesar_especifico();
	
	public function procesar()
	{
		try {
			$this->entorno()->db()->abrir_transaccion();
			
			$this->procesar_especifico();
			
			$this->entorno()->db()->cerrar_transaccion();
		} catch (error_db $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			$this->entorno()->log()->error(__METHOD__ . ": ERROR_DB: " . $e->getMessage());
			throw $e;
		} catch (\toba_error_db $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			$this->entorno()->log()->error(__METHOD__ . ": TOBA_ERROR_DB: " . $e->get_mensaje() . " SQL: " . $e->get_sql_ejecutado());
			throw $e;
		} catch (error_negocio $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			$this->entorno()->log()->error(__METHOD__ . ": ERROR_NEGOCIO: " . $e->get_mensaje());
			throw $e;
		} catch (error $e) {
			$this->entorno()->db()->abortar_transaccion($e->get_mensaje());
			$this->entorno()->log()->error(__METHOD__ . ": ERROR: " . $e->get_mensaje());
			throw $e;
		} catch (\Exception $e) {
			$this->entorno()->db()->abortar_transaccion($e->getMessage());
			$this->entorno()->log()->error(__METHOD__ . ": EXCEPTION: " . $e->getMessage());
			throw $e;
		}
	}

	function get_datos()
	{
		return $this->datos;
	}

	public function set_datos($datos)
	{
		$this->datos = $datos;
	}
	
	function get_id()
	{
		return $this->id;
	}
	
	function set_id($id)
	{
		$this->id = $id;
	}
	
	protected function validar_campos_obligatorios($datos, $campos_obligatorios, $prefijo_desc = '')
	{
		$error_obligatorios = false;
		foreach ($campos_obligatorios as $nombre_campo) {
			if (!in_array($nombre_campo, $datos, true)
				AND ( !isset($datos[$nombre_campo])
					OR $datos[$nombre_campo] === NULL)) {
				$error_obligatorios[] = $nombre_campo;
			}
		}
		if ($error_obligatorios) {
			throw new error_negocio($prefijo_desc . ": Los siguientes campos son obligatorios: '"
				. implode("', '", $error_obligatorios) . "'.");
		}
	}
	
	protected function validar_datos_cargados()
	{
		if (!isset($this->datos)){
			throw new error(__METHOD__ . ': no hay datos para procesar');
		}
	}

    protected function validar_id()
    {
        if (!isset($this->id)){
            throw new error(__METHOD__ . ': el id de transaccion es invalido');
        }
    }
	//----------- DEPRECATED
	
	protected $es_alta;
	
	function es_alta()
	{
		return $this->es_alta;
	}
	
	function set_es_alta($es_alta)
	{
		$this->es_alta = $es_alta;
	}
}
