<?php

namespace SIU\SQ\Lib\util\Documentos;

use SIU\SQ\Nucleo\sq_nucleo;
use SIU\SQ\Lib\entorno\configurador;

use SIU\DocumentosCli\DocumentosCli;
use SIU\DocumentosCli\Services\Documentos;
use SIU\DocumentosCli\Exceptions\AraiDocsException;
use SIU\DocumentosCli\Entities\Documento;
use SIU\DocumentosCli\Entities\Archivo;
use SIU\DocumentosCli\Entities\Autorizacion;
use SIU\DocumentosCli\Entities\Origen;

//use Enqueue\Fs\FsConnectionFactory;
use SIU\DocumentosCli\AsyncQueue\AsyncQueueWorker;

class AraiDocumentos
{
    /**
     * @var AraiDocumentos
     */
    protected static $doc_lib;

    public function __construct()
    {
    }

    /**
     * @return DocumentosCli
     * @throws \Exception
     */
    public static function getInstancia()
    {
        if (! isset(self::$doc_lib)) {
            $lib = new DocumentosCli();

            $param_config = new configurador();

            $config = [
                "host_arai"         => $param_config->sq_ini['AraiDocumentos']['HOST'],
                "usr_arai"          => $param_config->sq_ini['AraiDocumentos']['USER'],
                "pass_arai"         => $param_config->sq_ini['AraiDocumentos']['PASS'],
                "logs_dir"          => sq_nucleo::get_path() . '/toba/temp/',
                "queue_path"        => $param_config->sq_ini['AraiDocumentos']['QUEUE_PATH'],
                "sistema_origen"    => isset($param_config->sq_ini['AraiDocumentos']['SISTEMA_ORIGEN']) ? $param_config->sq_ini['AraiDocumentos']['SISTEMA_ORIGEN'] : null,
                "queue_temp_dir"    => $param_config->sq_ini['AraiDocumentos']['QUEUE_TEMP_DIR'],
                "db_queue"          => $param_config->sq_ini['AraiDocumentos']['DB_QUEUE'],
                "dbq_host"          => $param_config->sq_ini['AraiDocumentos']['DB_HOST'],
                "dbq_port"          => $param_config->sq_ini['AraiDocumentos']['DB_PORT'],
                "dbq_user"          => $param_config->sq_ini['AraiDocumentos']['DB_USER'],
                "dbq_password"      => $param_config->sq_ini['AraiDocumentos']['DB_PASS'],
                "dbq_table_name"    => $param_config->sq_ini['AraiDocumentos']['DB_TABLE'],
                "polling_interval"  => $param_config->sq_ini['AraiDocumentos']['POLLING_INVERVAL'],
                "rest_nucleo"       => $param_config->sq_ini['AraiDocumentos']['REST_NUCLEO']
            ];

            $lib->setConfig($config);
            self::$doc_lib = $lib;
        }

        return self::$doc_lib;
    }

    /**
     * @return \SIU\DocumentosCli\Services\Bandejas
     * @throws \Exception
     */
    public static function getBandejasService()
    {
        $servicios_bandejas = self::getInstancia()->getBandejasService();
        return $servicios_bandejas;
    }

    /**
     * @return \SIU\DocumentosCli\Services\Documentos
     * @throws \Exception
     */
    public static function getDocumentosService()
    {
        $servicios_documentos = self::getInstancia()->getDocumentosService();
        return $servicios_documentos;
    }

    public static function exportarDocumento($data)
    {
        $cont = self::getInstancia()->getContainter();
        $url = isset($cont['config']['rest_nucleo']) ? $cont['config']['rest_nucleo'] : '';

        $archivo = new Archivo();
        $archivo->setContenidoBase64($data['archivo']);

        $autorizacion = new Autorizacion();
        $autorizacion->setModelo(Autorizacion::MODELO_SIMPLE);
        $autorizacion->addPersona($data['autorizacion']['usuario'], Autorizacion::NIVEL_BASICA);

        $origen = new Origen();
        $origen->setIDUsuarioSSO($data['origen']['id_usuario_sso']); //Usuario logueado
        $origen->setIDInstalacion('sq_nucleo');
        $origen->setCuentaUsuario($data['origen']['cuenta_usuario']);

        $documento = new Documento();
        $documento->setTipo($data['documento']['tipo']);
        $documento->setTitulo($data['documento']['titulo']);
        $documento->setDescripcion($data['documento']['descripcion']);
        $documento->setIDTramite($data['documento']['id_tramite']);
        $documento->setURL($url);
        $documento->setArchivo($archivo);
        $documento->setAutorizacion($autorizacion);
        $documento->setOrigen($origen);

        $documento->setReferenciaInterna($data['documento']['referencia_interna']); // Nro
        $documento->setIdentificador($data['documento']['identificador']);
        $documento->setAtributos($data['documento']['atributos']);    // Opcional, atributos de busqueda

        $servicios_documentos = self::getInstancia()->getDocumentosService();
        return $servicios_documentos->postDocumento($documento);

        //$servicios_documentos->postDocumentoAsync($documento);
    }

    public static function getDocumento($uid_documento)
    {
        try {
            $servicios_documentos = self::getInstancia()->getDocumentosService();
            return $servicios_documentos->getDocumento($uid_documento);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getNotificacion($uid_notificacion)
    {
        try {
            $notificaciones_service = self::getInstancia()->getNotificacionesService();
            return $notificaciones_service->getNotificacion($uid_notificacion);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function downloadDocumento($uid_documento, $filtro = null)
    {
        try {
            $servicios_documentos = self::getInstancia()->getDocumentosService();
            return $servicios_documentos->downloadDocumento($uid_documento);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function vincularTramite($uid_documento, $data)
    {
        try {
            $servicios_documentos = self::getInstancia()->getDocumentosService();
            return $servicios_documentos->vincularTramite($uid_documento, $data);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getTramites($id_usuario_sso, $id_area = null)
    {
        try {
            $servicios_tramites = self::getInstancia()->getTramitesService();
            return $servicios_tramites->getTramites($id_usuario_sso);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getTramite($id_tramite)
    {
        try {
            $servicios_tramites = self::getInstancia()->getTramitesService();
            return $servicios_tramites->getTramite($id_tramite);
        } catch (\Exception $e) {
            return false;
        }
    }

    public static function getUsuarios($filtro)
    {
        try {
            $servicios_usuarios = self::getInstancia()->getUsuariosService();
            return $servicios_usuarios->getUsuarios($filtro);
        } catch (\Exception $e) {
            return false;
        }
    }
}