<?php

namespace SIU\SQ\Lib\util;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;

class mailer
{
	protected $php_mailer;
	protected $es_desarrollo = false;
	protected $email_envios_desarrollo;
	protected $envia_emails = true;
	
	public function __construct($host,$smtp_auth,$username,$password,$SMTPSecure,$port,$es_desarrollo = false, $envia_emails = true, $email_envios_desarrollo = '')
	{
		$this->php_mailer = new \PHPMailer;
		$this->php_mailer->isSMTP();                // Set mailer to use SMTP
		$this->php_mailer->isHTML(true);
		$this->php_mailer->Host = "$host;$host";    // Specify main and backup SMTP servers
		$this->php_mailer->SMTPAuth = $smtp_auth;   // Enable SMTP authentication
		$this->php_mailer->Username = $username;    // SMTP username
		$this->php_mailer->Password = $password;    // SMTP password
		$this->php_mailer->SMTPSecure = $SMTPSecure;// Enable TLS encryption ('tls'), `ssl` also accepted
		$this->php_mailer->Port = $port;            // TCP port to connect to
		$this->php_mailer->CharSet = 'utf-8';
		$this->es_desarrollo = $es_desarrollo;
		$this->envia_emails = $envia_emails;
		$this->email_envios_desarrollo = $email_envios_desarrollo;
	}
	
	function set_from($email,$nombre)
	{
		$this->php_mailer->setFrom($email, $nombre);
	}
	
	function set_destinatario($email,$nombre = null)
	{
		if ($this->es_desarrollo) {
			$destinatario = $this->email_envios_desarrollo;
		} else {

			$email = trim($email);

			if(!filter_var($email, FILTER_VALIDATE_EMAIL)){
				throw new error_negocio( 'El email ingresado no es valido. ' . $email);
			}

			$destinatario = $email;
		}
		$this->php_mailer->clearAddresses();//limpio destinatarios cargados anteriormente
		$this->php_mailer->addAddress($destinatario,$nombre);     // Add a recipient
	}
	
	function set_asunto($asunto)
	{
		$this->php_mailer->Subject = $asunto;
	}
	
	function adjuntar_archivo($path, $name, $encoding, $type)
	{
		$this->php_mailer->addAttachment($path, $name, $encoding, $type);
	}
    
    function eliminar_adjuntos()
	{
		$this->php_mailer->clearAttachments();
	}
	
	function set_cuerpo($cuerpo)
	{
		$this->php_mailer->Body = $cuerpo;//cuerpo es HTML
	}
	
	function set_smtp_auto_tls($smtp_auto_tls)
	{
		$this->php_mailer->SMTPAutoTLS = $smtp_auto_tls;
	}
	
	function enviar_email($asunto,$cuerpo,$destinatario_email,$destinatario_nombre = null)
	{
		if($this->envia_emails){
			if ($this->es_desarrollo) {
				$asunto = '[DESARROLLO] ' . $asunto;
				$asunto .= ' [PARA: ' . $destinatario_email . ']';
			}
			$this->set_asunto($asunto);
			$this->set_cuerpo($cuerpo);
			$this->set_destinatario($destinatario_email, $destinatario_nombre);
			$estado = $this->php_mailer->send();
			if( !$estado ){
				throw new error($this->php_mailer->ErrorInfo);
			}
		}
	}
	
}
