<?php
namespace SIU\SQ\Lib\util;

/**
 * Description of planificador_tareas
 *
 * @author leo
 */
class planificador_tareas
{
	protected $entorno;
	protected $procesos;
	protected $archivo_pid;

	//TIPOS DE FRECUENCIA
	const DIA_FIJO = 'DIA FIJO';
	const DIARIO = 'DIARIO';
	const INTERVALO = 'INTERVALO';
	const CLASS_PROCESO = "SIU\SQ\Lib\procesos\proceso";

	function __construct($archivo_pid, $entorno, $procesos)
	{
		$this->archivo_pid = $archivo_pid;
		$this->entorno = $entorno;
		$this->procesos = $procesos;
	}

	function ejecutar_tareas_planificadas()
	{
		// evitar concurrencia
		if (file_exists($this->archivo_pid)) {
			$this->entorno->log()->notice('ERROR. Existe un proceso en ejecución');
			throw new \Exception("Existe un proceso en ejecucion", 254);
		} else {
			$pid = getmypid();
			file_put_contents($this->archivo_pid, $pid);
			if (file_exists($this->archivo_pid)) {
				$this->entorno->log()->notice('OK. Comienza la búsqueda de procesos a ejecutar');
				try {
					$this->seleccionar_tareas_a_ejecutar();
				} catch (\Exception $ex) {
					throw $ex;
				} finally {
					unlink($this->archivo_pid);
				}
			} else {
				throw new \Exception("No pudo crearse el archivo sanaviron.pid. Verifique los permisos de escritura de la carpeta log.", 2);
			}
		}
	}

	function seleccionar_tareas_a_ejecutar()
	{
		try {
			$sql = "SELECT tp.id, tp.id_proceso, tp.tipo_frecuencia, tp.dia, tp.hora, tp.intervalo,
						CASE
							WHEN pr.prioridad = 'A' THEN 1
							WHEN pr.prioridad = 'M' THEN 2
							WHEN pr.prioridad = 'B' THEN 3
						END as prioridad
					FROM procesos_tareas_programadas tp, procesos pr
					WHERE tp.id_proceso = pr.id
					ORDER BY prioridad ASC;";
			$tareas = $this->entorno->db()->consultar($sql);
			foreach ($tareas as $tarea)
			{
				$query = "SELECT id_proceso, iniciado_en, finalizado_en, id_proceso_ejecucion_estado 
                        FROM procesos_ejecuciones 
                        WHERE id_proceso = " . $tarea['id_proceso'] . "
                        ORDER BY iniciado_en DESC LIMIT 1;";
				$ultima_ejecucion = $this->entorno->db()->consultar($query);

				if ($this->cumple_condicion_ejecucion($tarea, $ultima_ejecucion)) {
					if (isset($this->procesos[$tarea['id_proceso']]) && is_object($this->procesos[$tarea['id_proceso']]) 
							&& is_subclass_of($this->procesos[$tarea['id_proceso']], self::CLASS_PROCESO)) {
						$this->procesos[$tarea['id_proceso']]->ejecutar();	
					} else {
						$this->entorno->log()->addError('PLANIFICADOR DE TAREAS. El proceso con id ' . $tarea['id_proceso'] . ' no se encuentra configurado en el comando "tareas_planificadas"');
						throw new \Exception('PLANIFICADOR DE TAREAS. El proceso con id ' . $tarea['id_proceso'] . ' no se encuentra configurado en el comando "tareas_planificadas"');
					}
				}
			}
		} catch (\SIU\SQ\Lib\errores\error $e) {
			throw $e;
		} catch (\Exception $e) {
			throw $e;
		}
	}

	function cumple_condicion_ejecucion($tarea, $ultima_ejecucion)
	{
		$ahora = date_create_from_format('Y-m-d H:i:s.u', date('Y-m-d H:i:s.u'));
		$es_el_dia = $ahora->format('w');
		$hora_a_ejecutar = date_create_from_format('H:i', $tarea['hora']);
		$dia_a_ejecutar = $tarea['dia'];
		if (!empty($ultima_ejecucion)) {
			$hora_ultima_ejecucion = date_create_from_format('Y-m-d H:i:s.u', $ultima_ejecucion[0]['iniciado_en']);
			switch ($tarea['tipo_frecuencia']) {
				case self::DIA_FIJO:
					return $hora_a_ejecutar > $hora_ultima_ejecucion && $es_el_dia == $dia_a_ejecutar && $ahora >= $hora_a_ejecutar;
				case self::DIARIO:
					return $hora_a_ejecutar > $hora_ultima_ejecucion && $ahora >= $hora_a_ejecutar;
				case self::INTERVALO:
					$intervalo = new \DateInterval('PT' . $tarea['intervalo'] . 'M');
					return $ahora >= $hora_ultima_ejecucion->add($intervalo);
				default:
					break;
			}
		} else {
			switch ($tarea['tipo_frecuencia']) {
				case self::DIA_FIJO:
					return $es_el_dia == $dia_a_ejecutar && $ahora >= $hora_a_ejecutar;
				case self::DIARIO:
					return $ahora >= $hora_a_ejecutar;
				case self::INTERVALO:
					return true;
				default:
					break;
			}
		}
	}

}
