<?php
namespace SIU\SQ\Lib\util;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\lib;

// Este seteo solo afecta a la librería JavaBridge que esta tirando errores de DEPRECATED
ini_set("error_reporting", E_ALL & ~E_DEPRECATED);

// Para levantar el servidor jasper ejecutar
//$java -jar $TOBA_DIR/php/3ros/JavaBridge/WEB-INF/lib/JavaBridge.jar SERVLET:8081

class toba_vista_jasperreports_json extends \toba_vista_jasperreports
{
	protected $json_path;
	protected $qr_path;
	protected $datos_venta;
	protected $datos_qr;
	
	public function __construct($nombre_archivo="comprobante_")
	{
		$this->validar_instalacion_jasper();
		$this->temp_salida = \toba::proyecto()->get_path_temp().'/'.uniqid($nombre_archivo).'.pdf';
		$this->cargar_jasper();
		/*Creamos una variable tipo arreglo que contendrá los parámetros */
		$this->parametros = new \Java("java.util.HashMap");
	}
	
	function get_json_path()
	{
		return $this->json_path;
	}
	
	function get_file_path()
	{
		return $this->temp_salida;
	}
	
	function set_json($json_string)
	{
		//Creo un archivo JSON para guardar el contenido
		$nombre = \toba::proyecto()->get_path_temp().'/'.md5(uniqid(time()));
		\toba_manejador_archivos::crear_archivo_con_datos($nombre, $json_string);
		$this->json_path = $nombre;
		$this->xpath_data = null;
		$this->modo_archivo = true;
		$this->limpiar_modo_archivo = true;
	}
	
	function generar_pdf()
	{
		$this->validar_archivo_jasper();
		$this->set_valores_reporte();
		//Uno los distintos metareportes (jrprint) en un solo archivo para enviar el pdf.
		if (! $this->hay_metareportes()) {
			// Pego los datos al jasper y creo el jprint
			$this->completar_con_datos_json();
		}
		$this->crear_pdf();//Aca uno todos los jprint en uno solito		
		$this->borrar_archivos_auxiliares_creados();
	}
	
	function validar_instalacion_jasper()
	{
		$path_proyecto_toba = \toba::proyecto()->get_path();
		if(!file_exists("$path_proyecto_toba/../vendor/siu-toba/jasper/JavaBridge/java/Java.inc") ||
			!file_exists("$path_proyecto_toba/../vendor/siu-toba/jasper/JasperReports")
		){
			throw new error('Jasper no se encuentra instalado. '
				. 'Debe incluir las carpetas JavaBridge y JasperReports en el directorio '
				. '/vendor/jasper/ las mismas se pueden descargar de http://toba.siu.edu.ar/downloads/toba/JasperReports_JavaBridge.zip');
		}
	}
	
	function validar_instalacion_php_qr_code()
	{
		//valido que se haya incluido la carpeta phpqrcode
		$php_qr_code_file_path = \toba::proyecto()->get_path() . "/../vendor/aferrandini/phpqrcode/lib/PHPQRCode.php";
		if(file_exists($php_qr_code_file_path)){
			require_once($php_qr_code_file_path);
		}else{
			throw new error('PHP QR CODE  no se encuentra instalado.');
		}
	}
	
	function validar_archivo_jasper()
	{
		if(empty($this->path_reporte)){
			throw new error('Debe especificar el nombre del archivo .jasper que va a utilizar para la impresion');
		}
		if(!file_exists($this->path_reporte)){
			throw new error('Debe incluir el archivo ' . $this->path_reporte);
		}
	}
	
	function set_valores_reporte()
	{
		$this->set_json(json_encode($this->datos_venta, JSON_UNESCAPED_UNICODE));
		
		//QR
		if(!empty($this->datos_qr)){
			$this->validar_instalacion_php_qr_code();
			$path_pdf = $this->get_nombre_archivo_generado();
			$this->qr_path = substr($path_pdf, 0,strlen($path_pdf)-3) . "png";
			$tamanio = 3;
			$contenido_qr = json_encode($this->datos_qr, JSON_UNESCAPED_UNICODE);
			\PHPQRCode\QRcode::png($contenido_qr, $this->qr_path, 'L', 4, $tamanio);
			$this->set_parametro('qr', 'S', $this->qr_path);
		}
	}
	
	function completar_con_datos_json()
	{
		if (! isset($this->path_reporte)) {//Lo chequeo aca adentro por si la funcion se llama mas de una vez
			throw new \toba_error_def("Falta definir el .jasper con set_path_reporte");
		}
		
		if ($this->modo_archivo) {//Si el conjunto de datos viene de un archivo comun
			$jrl = new \JavaClass("net.sf.jasperreports.engine.util.JRLoader");
			$inputStream = $jrl->getLocationInputStream($this->json_path);
			$JsonQueryExecuterFactory = new \JavaClass("net.sf.jasperreports.engine.query.JsonQueryExecuterFactory");
			$this->parametros->put($JsonQueryExecuterFactory->JSON_INPUT_STREAM, $inputStream);
			try{
				$print = $this->jasper->fillReport($this->path_reporte, $this->parametros);
			} catch (\JavaException $ex) {
				$trace = new \Java("java.io.ByteArrayOutputStream");
				$ex->printStackTrace(new \Java("java.io.PrintStream", $trace));
				throw new error("Error al llenar el pdf. Trace: $trace\n");
			}
			
		}  else {//El conjunto de datos viene de una db o datasource
			if (! isset($this->conexion)) {
				$this->conexion = $this->instanciar_conexion_default();
			}
			if ($this->conexion instanceof toba_db) {//Si es una base toba, le configuro el schema
				$con1 = $this->configurar_bd($this->conexion);
			} else {
				$con1 = $this->conexion;
			}
			//Creo el reporte finalmente con la conexion JDBC
			$print = $this->jasper->fillReport($this->path_reporte, $this->parametros, $con1);
			$con1->close();
		}
		$this->agregar_metareporte($print);
	}
	
	/**
	 * Redefino este metodo con el unico objetivo de cambiar el path en el require_once
	 * @throws toba_error_usuario
	 */
	protected function cargar_jasper()
	{
		$jasper_host = '127.0.0.1';
		$jasper_port = '8081';
		
		if (getenv('JASPER_HOST') !== false) {
			$jasper_host = getenv('JASPER_HOST');
		}
		if (getenv('JASPER_PORT') !== false) {
			$jasper_port = getenv('JASPER_PORT');
		}
		if( !defined("JAVA_HOSTS")) define("JAVA_HOSTS", "$jasper_host:$jasper_port");
		
		//Incluimos la libreria JavaBridge
		$path_proyecto_toba = \toba::proyecto()->get_path();
		require_once($path_proyecto_toba."/../vendor/siu-toba/jasper/JavaBridge/java/Java.inc");
		
		//Creamos una variable que va a contener todas las librerías java presentes
		$path_libs = $path_proyecto_toba.'/../vendor/siu-toba/jasper/JasperReports';
		$classpath = '';
		try {
			$archivos = \toba_manejador_archivos::get_archivos_directorio($path_libs, '|.*\.jar$|' , true);
			foreach ($archivos as $archivo) {
				$classpath .= "file:$archivo;" ;
			}
		} catch (\toba_error $et) {
			\toba::logger()->error($et->getMessage());		//No se encontro el directorio, asi que no agrega nada al path y sigue el comportamiento que tenia con opendir
		}
		try {
			//Añadimos las librerías
			java_require($classpath);
			
			//Creamos el objeto JasperReport que permite obtener el reporte
			$this->jasper = new \JavaClass("net.sf.jasperreports.engine.JasperFillManager");
			
		} catch (\JavaException $ex) {
			$trace = new \Java("java.io.ByteArrayOutputStream");
			$ex->printStackTrace(new \Java("java.io.PrintStream", $trace));
			print "java stack trace: $trace\n";
		} catch (\java_ConnectException $e) {
			\toba::logger()->error($e->getMessage());
			throw new \toba_error_usuario( 'No es posible generar el reporte, el servlet Jasper no se encuentra corriendo');
		}
	}
	
	function borrar_archivos_auxiliares_creados()
	{
		//borro el archivo json
		unlink($this->get_json_path());
		//borro el png con el codigo qr
		if(!empty($this->datos_qr)){
			unlink($this->qr_path);
		}
	}
	
	//------------------------------------------------------------------------
	//-- Envio del archivo al cliente
	//------------------------------------------------------------------------
	
	function enviar_archivo_sin_eliminarlo()//si se quiere enviar y eliminar usar $this->enviar_archivo();
	{
		$this->cabecera_http(filesize($this->temp_salida));
		readfile($this->temp_salida);
	}
	
	//------------------------------------------------------------------------
	//-- getters & setters
	//------------------------------------------------------------------------
	
	function set_datos_venta($datos_venta)
	{
		$this->datos_venta = $datos_venta;
	}
	
	function set_datos_qr(array $datos_qr)
	{
		$this->datos_qr = $datos_qr;
	}
	
	
	
}
